@@ -68,7 +68,7 @@ class TestCapabilities(unittest.TestCase):
             ( 'i686',   'xen',
               [['xen', None, []]], { 'pae': capabilities.FEATURE_ON } ),
             ( 'i686',   'hvm',
-              [['xen', "/usr/lib64/xen/bin/qemu-dm", ['pc', 'isapc']]], { 'pae': capabilities.FEATURE_ON|capabilities.FEATURE_OFF } ),
+              [['xen', "/usr/lib64/xen/bin/qemu-dm", ['pc', 'isapc']]], { 'pae': capabilities.FEATURE_ON | capabilities.FEATURE_OFF } ),
             ( 'x86_64', 'hvm',
               [['xen', "/usr/lib64/xen/bin/qemu-dm", ['pc', 'isapc']]], {} )
         ]
@@ -81,17 +81,18 @@ class TestCapabilities(unittest.TestCase):
 
         guests = [
             ( 'x86_64', 'hvm',
-              [['qemu','/usr/bin/qemu-system-x86_64', ['pc', 'isapc']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-x86_64', ['pc', 'isapc']]], {} ),
             ( 'i686',   'hvm',
-              [['qemu','/usr/bin/qemu', ['pc', 'isapc']]], {} ),
+              [['qemu', '/usr/bin/qemu', ['pc', 'isapc']]], {} ),
             ( 'mips',   'hvm',
-              [['qemu','/usr/bin/qemu-system-mips', ['mips']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-mips', ['mips']]], {} ),
             ( 'mipsel', 'hvm',
-              [['qemu','/usr/bin/qemu-system-mipsel', ['mips']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-mipsel', ['mips']]], {} ),
             ( 'sparc',  'hvm',
-              [['qemu','/usr/bin/qemu-system-sparc', ['sun4m']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-sparc', ['sun4m']]], {} ),
             ( 'ppc',    'hvm',
-              [['qemu','/usr/bin/qemu-system-ppc', ['g3bw', 'mac99', 'prep']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-ppc',
+               ['g3bw', 'mac99', 'prep']]], {} ),
         ]
 
         self._testCapabilities("capabilities-qemu.xml", host, guests, secmodel)
@@ -101,30 +102,31 @@ class TestCapabilities(unittest.TestCase):
 
         guests = [
             ( 'i686',   'hvm',
-              [['qemu','/usr/bin/qemu', ['pc', 'isapc']],
+              [['qemu', '/usr/bin/qemu', ['pc', 'isapc']],
                ['kvm', '/usr/bin/qemu-kvm', ['pc', 'isapc']]], {} ),
             ( 'x86_64', 'hvm',
-              [['qemu','/usr/bin/qemu-system-x86_64', ['pc', 'isapc']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-x86_64', ['pc', 'isapc']]], {} ),
             ( 'mips',   'hvm',
-              [['qemu','/usr/bin/qemu-system-mips', ['mips']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-mips', ['mips']]], {} ),
             ( 'mipsel', 'hvm',
-              [['qemu','/usr/bin/qemu-system-mipsel', ['mips']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-mipsel', ['mips']]], {} ),
             ( 'sparc',  'hvm',
-              [['qemu','/usr/bin/qemu-system-sparc', ['sun4m']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-sparc', ['sun4m']]], {} ),
             ( 'ppc',    'hvm',
-              [['qemu','/usr/bin/qemu-system-ppc', ['g3bw', 'mac99', 'prep']]], {} ),
+              [['qemu', '/usr/bin/qemu-system-ppc',
+               ['g3bw', 'mac99', 'prep']]], {} ),
         ]
 
         self._testCapabilities("capabilities-kvm.xml", host, guests)
 
     def testCapabilities4(self):
         host = ( 'i686',
-                 { 'pae': capabilities.FEATURE_ON|capabilities.FEATURE_OFF })
+                 { 'pae': capabilities.FEATURE_ON | capabilities.FEATURE_OFF })
 
         guests = [
             ( 'i686', 'linux',
               [['test', None, []]],
-              { 'pae': capabilities.FEATURE_ON|capabilities.FEATURE_OFF } ),
+              { 'pae': capabilities.FEATURE_ON | capabilities.FEATURE_OFF } ),
         ]
 
         self._testCapabilities("capabilities-test.xml", host, guests)
@@ -167,6 +169,7 @@ class TestCapabilities(unittest.TestCase):
             self.assertEquals(caps.host.features[f], feature_dict[f])
 
         self.assertEquals(caps.host.cpu.model, "core2duo")
+        self.assertEquals(caps.host.cpu.vendor, "Intel")
         self.assertEquals(caps.host.cpu.threads, "3")
         self.assertEquals(caps.host.cpu.cores, "5")
         self.assertEquals(caps.host.cpu.sockets, "7")
@@ -196,5 +199,53 @@ class TestCapabilities(unittest.TestCase):
         test_utils(rhel_kvm_caps, False, True, True, False, False)
         test_utils(new_caps_no_kvm, False, True, False, False, False)
 
+    def testCPUMap(self):
+        caps = self._buildCaps("libvirt-0.7.6-qemu-caps.xml")
+        cpu_64 = caps.get_cpu_values("x86_64")
+        cpu_32 = caps.get_cpu_values("i486")
+        cpu_random = caps.get_cpu_values("mips")
+
+        def test_cpu_map(cpumap, vendors, features, cpus):
+            cpunames = sorted(map(lambda c: c.model, cpumap.cpus),
+                              key=str.lower)
+
+            self.assertEquals(cpumap.vendors, vendors)
+            self.assertEquals(cpumap.features, features)
+            self.assertEquals(cpunames, cpus)
+
+        def test_single_cpu(cpumap, model, vendor, features):
+            cpu = cpumap.get_cpu(model)
+            self.assertEquals(cpu.vendor, vendor)
+            self.assertEquals(cpu.features, features)
+
+        self.assertEquals(cpu_64, cpu_32)
+
+        x86_vendors = ["AMD", "Intel"]
+        x86_features = [
+            '3dnow', '3dnowext', '3dnowprefetch', 'abm', 'acpi', 'apic',
+            'cid', 'clflush', 'cmov', 'cmp_legacy', 'cr8legacy', 'cx16',
+            'cx8', 'dca', 'de', 'ds', 'ds_cpl', 'est', 'extapic', 'fpu',
+            'fxsr', 'fxsr_opt', 'ht', 'hypervisor', 'ia64', 'lahf_lm', 'lm',
+            'mca', 'mce', 'misalignsse', 'mmx', 'mmxext', 'monitor', 'msr',
+            'mtrr', 'nx', 'osvw', 'pae', 'pat', 'pbe', 'pdpe1gb', 'pge', 'pn',
+            'pni', 'popcnt', 'pse', 'pse36', 'rdtscp', 'sep', 'skinit', 'ss',
+            'sse', 'sse2', 'sse4.1', 'sse4.2', 'sse4a', 'ssse3', 'svm',
+            'syscall', 'tm', 'tm2', 'tsc', 'vme', 'vmx', 'wdt', 'x2apic',
+            'xtpr']
+        x86_cpunames = [
+            '486', 'athlon', 'Conroe', 'core2duo', 'coreduo', 'n270',
+            'Nehalem', 'Opteron_G1', 'Opteron_G2', 'Opteron_G3', 'Penryn',
+            'pentium', 'pentium2', 'pentium3', 'pentiumpro', 'phenom',
+            'qemu32', 'qemu64']
+
+        test_cpu_map(cpu_64, x86_vendors, x86_features, x86_cpunames)
+        test_cpu_map(cpu_random, [], [], [])
+
+        athlon_features = [
+            '3dnow', '3dnowext', 'apic', 'cmov', 'cx8', 'de', 'fpu', 'fxsr',
+            'mce', 'mmx', 'mmxext', 'msr', 'mtrr', 'pae', 'pat', 'pge', 'pse',
+            'pse36', 'sep', 'sse', 'sse2', 'tsc', 'vme']
+        test_single_cpu(cpu_64, "athlon", "AMD", athlon_features)
+
 if __name__ == "__main__":
     unittest.main()
