@@ -15,40 +15,57 @@
 # MA 02110-1301 USA.
 
 import commands
-import os, sys
+import os
+import sys
+
+import utils
 
-# Set DISPLAY if it isn't already set
-os.environ["DISPLAY"] = "testdisplay"
 os.environ["VIRTCONV_TEST_NO_DISK_CONVERSION"] = "1"
+os.environ["LANG"] = "en_US.UTF-8"
 
 testuri = "test:///`pwd`/tests/testdriver.xml"
 
 # There is a hack in virtinst/cli.py to find this magic string and
 # convince virtinst we are using a remote connection.
-remoteuri = "__virtinst_test_remote__test:///`pwd`/tests/testdriver.xml"
+fakeuri     = "__virtinst_test__" + testuri + ",predictable"
+remoteuri   = fakeuri + ",remote"
+kvmuri      = fakeuri + ",caps=`pwd`/tests/capabilities-xml/libvirt-0.7.6-qemu-caps.xml,qemu"
+xenuri      = fakeuri + ",caps=`pwd`/tests/capabilities-xml/rhel5.4-xen-caps-virt-enabled.xml,xen"
+xenia64uri  = fakeuri + ",caps=`pwd`/tests/capabilities-xml/xen-ia64-hvm.xml,xen"
 
 # Location
+image_prefix = "/tmp/__virtinst_cli_"
 xmldir = "tests/cli-test-xml"
 treedir = "%s/faketree" % xmldir
 vcdir = "%s/virtconv" % xmldir
-ro_dir = "clitest_rodir"
+ro_dir = image_prefix + "clitest_rodir"
 ro_img = "%s/cli_exist3ro.img" % ro_dir
 ro_noexist_img = "%s/idontexist.img" % ro_dir
+compare_xmldir = "%s/compare" % xmldir
 virtconv_out = "virtconv-outdir"
 
 # Images that will be created by virt-install/virt-clone, and removed before
 # each run
-new_images =    [ "cli_new1.img", "cli_new2.img", "cli_new3.img",
-                  "cli_exist1-clone.img", "cli_exist2-clone.img"]
+new_images = [
+    image_prefix + "new1.img",
+    image_prefix + "new2.img",
+    image_prefix + "new3.img",
+    image_prefix + "exist1-clone.img",
+    image_prefix + "exist2-clone.img",
+]
 
 # Images that are expected to exist before a command is run
-exist_images =  [ "cli_exist1.img", "cli_exist2.img", ro_img]
+exist_images = [
+    image_prefix + "exist1.img",
+    image_prefix + "exist2.img",
+    ro_img,
+]
 
 # Images that need to exist ahead of time for virt-image
-virtimage_exist = [os.path.join(xmldir, "cli_root.raw")]
+virtimage_exist = ["/tmp/__virtinst__cli_root.raw"]
 
 # Images created by virt-image
-virtimage_new = [os.path.join(xmldir, "cli_scratch.raw")]
+virtimage_new = ["/tmp/__virtinst__cli_scratch.raw"]
 
 # virt-convert output dirs
 virtconv_dirs = [virtconv_out]
@@ -61,6 +78,9 @@ clean_files = (new_images + exist_images +
 test_files = {
     'TESTURI'           : testuri,
     'REMOTEURI'         : remoteuri,
+    'KVMURI'            : kvmuri,
+    'XENURI'            : xenuri,
+    'XENIA64URI'        : xenia64uri,
     'CLONE_DISK_XML'    : "%s/clone-disk.xml" % xmldir,
     'CLONE_STORAGE_XML' : "%s/clone-disk-managed.xml" % xmldir,
     'CLONE_NOEXIST_XML' : "%s/clone-disk-noexist.xml" % xmldir,
@@ -75,6 +95,7 @@ test_files = {
     'POOL'              : "default-pool",
     'VOL'               : "testvol1.img",
     'DIR'               : os.getcwd(),
+    'TREEDIR'           : treedir,
     'MANAGEDEXIST1'     : "/default-pool/testvol1.img",
     'MANAGEDEXIST2'     : "/default-pool/testvol2.img",
     'MANAGEDNEW1'       : "/default-pool/clonevol",
@@ -85,6 +106,7 @@ test_files = {
 
     'VIRTCONV_OUT'      : "%s/test.out" % virtconv_out,
     'VC_IMG1'           : "%s/virtimage/test1.virt-image" % vcdir,
+    'VC_IMG2'           : "tests/image-xml/image-format.xml",
     'VMX_IMG1'          : "%s/vmx/test1.vmx" % vcdir,
 }
 
@@ -97,16 +119,16 @@ CLI test matrix
 Format:
 
 "appname" {
-  "global_args" : Arguments to be passed to every app invocation
+  "globalargs" : Arguments to be passed to every app invocation
 
   "categoryfoo" : { Some descriptive test catagory name (e.g. storage)
 
-    "categoryfoo_args" : Args to be applied to all invocations in category
+    "args" : Args to be applied to all invocations in category
 
     "valid" : { # Argument strings that should succeed
       "--option --string --number1" # Some option string to test. The
           resulting cmdstr would be:
-          $ appname global_args categoryfoo_args --option --string --number1
+          $ appname globalargs categoryfoo_args --option --string --number1
     }
 
     "invalid" : { # Argument strings that should fail
@@ -125,10 +147,10 @@ args_dict = {
 
 
   "virt-install" : {
-    "global_args" : " --connect %(TESTURI)s -d --name foobar --ram 64",
+    "globalargs" : " --connect %(TESTURI)s -d --name foobar --ram 64",
 
     "storage" : {
-      "storage_args": "--pxe --nographics --noautoconsole --hvm",
+      "args": "--pxe --nographics --noautoconsole --hvm",
 
       "valid"  : [
         # Existing file, other opts
@@ -142,6 +164,8 @@ args_dict = {
 
         # Existing disk, lots of opts
         "--disk path=%(EXISTIMG1)s,perms=ro,size=.0001,cache=writethrough",
+        # Existing disk, rw perms
+        "--disk path=%(EXISTIMG1)s,perms=rw",
         # Existing floppy
         "--disk path=%(EXISTIMG1)s,device=floppy",
         # Existing disk, no extra options
@@ -180,6 +204,8 @@ args_dict = {
         "--disk %(EXISTIMG1)s,driver_name=qemu,driver_type=qcow2",
         # Unknown driver name and type options
         "--disk %(EXISTIMG1)s,driver_name=foobar,driver_type=foobaz",
+        # Using a storage pool source as a disk
+        "--disk /dev/hda",
       ],
 
       "invalid": [
@@ -217,7 +243,7 @@ args_dict = {
      }, # category "storage"
 
      "install" : {
-      "install_args": "--nographics --noautoconsole --nodisks",
+      "args": "--nographics --noautoconsole --nodisks",
 
       "valid" : [
         # Simple cdrom install
@@ -233,19 +259,19 @@ args_dict = {
         # Convert i*86 -> i686
         "--arch i486 --pxe",
         # Directory tree URL install
-        "--hvm --location %s" % treedir,
+        "--hvm --location %(TREEDIR)s",
         # Directory tree URL install with extra-args
-        "--hvm --location %s --extra-args console=ttyS0" % treedir,
+        "--hvm --location %(TREEDIR)s --extra-args console=ttyS0",
         # Directory tree CDROM install
-        "--hvm --cdrom %s" % treedir,
+        "--hvm --cdrom %(TREEDIR)s",
         # Paravirt location
-        "--paravirt --location %s" % treedir,
+        "--paravirt --location %(TREEDIR)s",
         # Using ro path as a cd media
         "--hvm --cdrom %(ROIMG)s",
         # Paravirt location with --os-variant none
-        "--paravirt --location %s --os-variant none" % treedir,
+        "--paravirt --location %(TREEDIR)s --os-variant none",
         # URL install with manual os-variant
-        "--hvm --location %s --os-variant fedora12" % treedir,
+        "--hvm --location %(TREEDIR)s --os-variant fedora12",
         # Boot menu
         "--hvm --pxe --boot menu=on",
         # Kernel params
@@ -277,15 +303,25 @@ args_dict = {
      }, # category "install"
 
      "graphics": {
-      "graphics_args": "--noautoconsole --nodisks --pxe",
+      "args": "--noautoconsole --nodisks --pxe",
 
       "valid": [
         # SDL
         "--sdl",
+        # --graphics SDL
+        "--graphics sdl",
+        # --graphics none,
+        "--graphics none",
         # VNC w/ lots of options
         "--vnc --keymap ja --vncport 5950 --vnclisten 1.2.3.4",
+        # VNC w/ lots of options, new way
+        "--graphics vnc,port=5950,listen=1.2.3.4,keymap=ja,password=foo",
+        # SPICE w/ lots of options
+        "--graphics spice,port=5950,tlsport=5950,listen=1.2.3.4,keymap=ja",
         # --video option
         "--vnc --video vga",
+        # --video option
+        "--graphics spice --video qxl",
         # --keymap local,
         "--vnc --keymap local",
         # --keymap none
@@ -297,14 +333,20 @@ args_dict = {
         "--vnc --keymap ZZZ",
         # Invalid port
         "--vnc --vncport -50",
+        # Invalid port
+        "--graphics spice,tlsport=-50",
         # Invalid --video
         "--vnc --video foobar",
+        # --graphics bogus
+        "--graphics vnc,foobar=baz",
+        # mixing old and new
+        "--graphics vnc --vnclisten 1.2.3.4",
       ],
 
      }, # category "graphics"
 
     "char" : {
-     "char_args": "--hvm --nographics --noautoconsole --nodisks --pxe",
+     "args": "--hvm --nographics --noautoconsole --nodisks --pxe",
 
      "valid": [
         # Simple devs
@@ -348,7 +390,7 @@ args_dict = {
      }, # category 'char'
 
      "cpuram" : {
-      "cpuram_args" : "--hvm --nographics --noautoconsole --nodisks --pxe",
+      "args" : "--hvm --nographics --noautoconsole --nodisks --pxe",
 
       "valid" : [
         # Max VCPUS
@@ -361,6 +403,18 @@ args_dict = {
         "--vcpus 4 --cpuset=auto",
         # Ram overcommit
         "--ram 100000000000",
+        # maxvcpus
+        "--vcpus 5,maxvcpus=10",
+        # Topology
+        "--vcpus 4,cores=2,threads=2,sockets=2",
+        # Topology auto-fill
+        "--vcpus 4,cores=1",
+        # Topology only
+        "--vcpus sockets=2,threads=2",
+        # Simple --cpu
+        "--cpu somemodel",
+        # Crazy --cpu
+        "--cpu foobar,+x2apic,+x2apicagain,-distest,forbid=foo,forbid=bar,disable=distest2,optional=opttest,require=reqtest,match=strict,vendor=meee",
       ],
 
       "invalid" : [
@@ -370,12 +424,20 @@ args_dict = {
         "--vcpus 32 --cpuset=autofoo",
         # Over max vcpus
         "--vcpus 10000",
+        # Over host vcpus w/ --check-cpu
+        "--vcpus 20 --check-cpu",
+        # maxvcpus less than cpus
+        "--vcpus 5,maxvcpus=1",
+        # vcpus unknown option
+        "--vcpus foo=bar",
+        # --cpu host, but no host CPU in caps
+        "--cpu host",
       ],
 
     }, # category 'cpuram'
 
      "misc": {
-      "misc_args": "--nographics --noautoconsole",
+      "args": "--nographics --noautoconsole",
 
       "valid": [
         # Specifying cdrom media via --disk
@@ -384,6 +446,8 @@ args_dict = {
         "--hvm --import --disk path=virt-install",
         # PV Import install
         "--paravirt --import --disk path=virt-install",
+        # PV Import install, print single XML
+        "--paravirt --import --disk path=virt-install --print-xml",
         # Import a floppy disk
         "--hvm --import --disk path=virt-install,device=floppy",
         # --autostart flag
@@ -392,6 +456,8 @@ args_dict = {
         "--hvm --nodisks --pxe --description \"foobar & baz\"",
         # HVM windows install with disk
         "--hvm --cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0",
+        # HVM windows install, print 3rd stage XML
+        "--hvm --cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0 --print-step 3",
         # --watchdog dev default
         "--hvm --nodisks --pxe --watchdog default",
         # --watchdog opts
@@ -411,6 +477,8 @@ args_dict = {
       ],
 
       "invalid": [
+        # Positional arguments error
+        "--hvm --nodisks --pxe foobar",
         # pxe and nonetworks
         "--nodisks --pxe --nonetworks",
         # Colliding name
@@ -421,11 +489,25 @@ args_dict = {
         "--hvm --nodisks --pxe --soundhw default --soundhw foobar",
         # Busted --security
         "--hvm --nodisks --pxe --security type=foobar",
+        # PV Import install, no second XML step
+        "--paravirt --import --disk path=virt-install --print-step 2",
+        # 2 stage install with --print-xml
+        "--hvm --nodisks --pxe --print-xml",
       ],
+
+      "compare": [
+        # No arguments
+        ("", "noargs-fail"),
+        # Diskless PXE install
+        ("--hvm --nodisks --pxe --print-step all", "simple-pxe"),
+        # HVM windows install with disk
+        ("--hvm --cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0 --vcpus cores=4", "w2k3-cdrom"),
+      ],
+
      }, # category "misc"
 
      "network": {
-      "network_args": "--pxe --nographics --noautoconsole --nodisks",
+      "args": "--pxe --nographics --noautoconsole --nodisks",
 
       "valid": [
         # Just a macaddr
@@ -465,7 +547,7 @@ args_dict = {
      }, # category "network"
 
      "hostdev" : {
-      "hostdev_args": "--noautoconsole --nographics --nodisks --pxe",
+      "args": "--noautoconsole --nographics --nodisks --pxe",
 
       "valid" : [
         # Host dev by libvirt name
@@ -485,7 +567,7 @@ args_dict = {
      }, # category "hostdev"
 
      "remote" : {
-      "remote_args": "--connect %(REMOTEURI)s --nographics --noautoconsole",
+      "args": "--connect %(REMOTEURI)s --nographics --noautoconsole",
 
       "valid" : [
         # Simple pxe nodisks
@@ -508,6 +590,83 @@ args_dict = {
 
      }, # category "remote"
 
+
+"kvm" : {
+  "args": "--connect %(KVMURI)s --noautoconsole",
+
+  "valid" : [
+    # HVM windows install with disk
+    "--cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0 --sound",
+    # F14 Directory tree URL install with extra-args
+    "--os-variant fedora14 --file %(EXISTIMG1)s --location %(TREEDIR)s --extra-args console=ttyS0 --sound"
+  ],
+
+  "invalid" : [
+    # Unknown machine type
+    "--nodisks --boot network --machine foobar",
+    # Invalid domain type for arch
+    "--nodisks --boot network --arch mips --virt-type kvm",
+    # Invalid arch/virt combo
+    "--nodisks --boot network --paravirt --arch mips",
+  ],
+
+  "compare" : [
+    # F14 Directory tree URL install with extra-args
+    ("--os-variant fedora14 --file %(EXISTIMG1)s --location %(TREEDIR)s --extra-args console=ttyS0 --cpu host", "kvm-f14-url"),
+    # Quiet URL install should make no noise
+    ("--os-variant fedora14 --disk %(NEWIMG1)s,size=.01 --location %(TREEDIR)s --extra-args console=ttyS0 --quiet", "quiet-url"),
+    # HVM windows install with disk
+    ("--cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0 --sound", "kvm-win2k3-cdrom"),
+
+    # xenner
+    ("--os-variant fedora14 --nodisks --boot hd --paravirt", "kvm-xenner"),
+    # plain qemu
+    ("--os-variant fedora14 --nodisks --boot cdrom --virt-type qemu",
+     "qemu-plain"),
+    # 32 on 64
+    ("--os-variant fedora14 --nodisks --boot network --nographics --arch i686",
+     "qemu-32-on-64"),
+    # kvm machine type 'pc'
+    ("--os-variant fedora14 --nodisks --boot fd --graphics spice --machine pc", "kvm-machine"),
+    # exotic arch + machine type
+    ("--os-variant fedora14 --nodisks --boot fd --graphics sdl --arch sparc --machine SS-20",
+     "qemu-sparc"),
+  ],
+
+}, # category "kvm"
+
+"xen" : {
+  "args": "--connect %(XENURI)s --noautoconsole",
+
+  "valid"   : [
+    # HVM
+    "--nodisks --cdrom %(EXISTIMG1)s --livecd --hvm",
+    # PV
+    "--nodisks --boot hd --paravirt",
+    # 32 on 64 xen
+    "--nodisks --boot hd --paravirt --arch i686",
+  ],
+
+  "invalid" : [
+  ],
+
+  "compare" : [
+    # Xen default
+    ("--disk %(EXISTIMG1)s --import", "xen-default"),
+    # Xen PV
+    ("--disk %(EXISTIMG1)s --location %(TREEDIR)s --paravirt", "xen-pv"),
+    # Xen HVM
+    ("--disk %(EXISTIMG1)s --cdrom %(EXISTIMG1)s --livecd --hvm", "xen-hvm"),
+    # ia64 default
+    ("--connect %(XENIA64URI)s --disk %(EXISTIMG1)s --import",
+     "xen-ia64-default"),
+    # ia64 pv
+    ("--connect %(XENIA64URI)s --disk %(EXISTIMG1)s --location %(TREEDIR)s --paravirt", "xen-ia64-pv"),
+    # ia64 hvm
+    ("--connect %(XENIA64URI)s --disk %(EXISTIMG1)s --location %(TREEDIR)s --hvm", "xen-ia64-hvm"),
+  ],
+
+},
     "prompt" : [ " --connect %(TESTURI)s --debug --prompt" ]
   },
 
@@ -515,10 +674,10 @@ args_dict = {
 
 
   "virt-clone": {
-    "global_args" : " --connect %(TESTURI)s -d",
+    "globalargs" : " --connect %(TESTURI)s -d",
 
     "general" : {
-      "general_args": "-n clonetest",
+      "args": "-n clonetest",
 
       "valid"  : [
         # Nodisk guest
@@ -543,6 +702,8 @@ args_dict = {
       ],
 
       "invalid": [
+        # Positional arguments error
+        "-o test foobar",
         # Non-existent vm name
         "-o idontexist",
         # Non-existent vm name with auto flag,
@@ -567,7 +728,7 @@ args_dict = {
      }, # category "general"
 
     "misc" : {
-      "misc_args": "",
+      "args": "",
 
       "valid" : [
         # Auto flag, no storage
@@ -585,11 +746,17 @@ args_dict = {
         "--auto-clone"
         # Auto flag, actual VM, without state skip
         "-o test-for-clone --auto-clone",
-      ]
+      ],
+
+      "compare" : [
+        ("-o test-for-clone --auto-clone --clone-running", "clone-auto1"),
+        ("-o test-clone-simple --name newvm --auto-clone --clone-running",
+         "clone-auto2"),
+      ],
     }, # category "misc"
 
      "remote" : {
-      "remote_args": "--connect %(REMOTEURI)s",
+      "args": "--connect %(REMOTEURI)s",
 
       "valid"  : [
         # Auto flag, no storage
@@ -603,22 +770,25 @@ args_dict = {
       ],
     }, # categort "remote"
 
-    "prompt" : [ " --connect %(TESTURI) --debug --prompt",
-                 " --connect %(TESTURI) --debug --original-xml %(CLONE_DISK_XML)s --prompt" ]
+    "prompt" : [
+        " --connect %(TESTURI) --debug --prompt",
+        " --connect %(TESTURI) --debug --original-xml %(CLONE_DISK_XML)s --prompt" ]
   }, # app 'virt-clone'
 
 
 
 
   'virt-image': {
-    "global_args" : " --connect %(TESTURI)s -d %(IMAGE_XML)s",
+    "globalargs" : " --connect %(TESTURI)s -d %(IMAGE_XML)s",
 
     "general" : {
-      "general_args" : "--name test-image",
+      "args" : "--name test-image",
 
       "valid": [
         # All default values
         "",
+        # Print default
+        "--print",
         # Manual boot idx 0
         "--boot 0",
         # Manual boot idx 1
@@ -636,7 +806,7 @@ args_dict = {
      }, # category 'general'
 
     "graphics" : {
-      "graphics_args" : "--name test-image --boot 0",
+      "args" : "--name test-image --boot 0",
 
       "valid": [
         # SDL
@@ -649,7 +819,7 @@ args_dict = {
     },
 
     "misc": {
-     "misc_args" : "",
+     "args" : "",
 
       "valid" : [
         # Colliding VM name w/ --replace
@@ -662,10 +832,16 @@ args_dict = {
         "--name test",
       ],
 
+      "compare" : [
+        ("--name foobar --ram 64 --os-variant winxp --boot 0", "image-boot0"),
+        ("--name foobar --ram 64 --network user,model=e1000 --boot 1",
+         "image-boot1"),
+      ]
+
      }, # category 'misc'
 
      "network": {
-      "network_args": "--name test-image --boot 0 --nographics",
+      "args": "--name test-image --boot 0 --nographics",
 
       "valid": [
         # user networking
@@ -695,10 +871,10 @@ args_dict = {
 
 
   "virt-convert" : {
-    "global_args" : "--debug",
+    "globalargs" : "--debug",
 
     "misc" : {
-     "misc_args": "",
+     "args": "",
 
      "valid": [
         # virt-image to default (virt-image) w/ no convert
@@ -715,6 +891,8 @@ args_dict = {
         "%(VC_IMG1)s -o vmx -D qcow2 %(VIRTCONV_OUT)s",
         # vmx to vmx no convert
         "%(VMX_IMG1)s -o vmx -D none %(VIRTCONV_OUT)s",
+        # virt-image with exotic formats specified
+        "%(VC_IMG2)s -o vmx -D vmdk %(VIRTCONV_OUT)s"
      ],
 
      "invalid": [
@@ -742,6 +920,10 @@ def runcomm(comm):
             print ret[1]
             print "\n"
 
+        if not debug:
+            sys.stdout.write(".")
+            sys.stdout.flush()
+
         return ret
     except Exception, e:
         return (-1, str(e))
@@ -757,6 +939,7 @@ def assertPass(comm):
                              "Command was: %s\n" % (comm) + \
                              "Error code : %d\n" % ret[0] + \
                              "Output was:\n%s" % ret[1])
+    return ret
 
 def assertFail(comm):
     ret = runcomm(comm)
@@ -765,9 +948,11 @@ def assertFail(comm):
                              "Command was: %s\n" % (comm) + \
                              "Error code : %d\n" % ret[0] + \
                              "Output was:\n%s" % ret[1])
+    return ret
+
 
 # Setup: build cliarg dict, which uses
-def run_tests(do_app):
+def run_tests(do_app, do_category):
     if do_app and do_app not in args_dict.keys():
         raise ValueError("Unknown app '%s'" % do_app)
 
@@ -777,19 +962,19 @@ def run_tests(do_app):
 
         unique = {}
         prompts = []
-        global_args = ""
+        globalargs = ""
 
         # Build default command line dict
         for option in args_dict.get(app):
-            if option == "global_args":
-                global_args = args_dict[app][option]
+            if option == "globalargs":
+                globalargs = args_dict[app][option]
                 continue
             elif option == "prompt":
                 prompts = args_dict[app][option]
                 continue
 
             # Default is a unique cmd string
-            unique[option]= args_dict[app][option]
+            unique[option] = args_dict[app][option]
 
         # Build up prompt cases
         if testprompt:
@@ -798,27 +983,75 @@ def run_tests(do_app):
                 run_prompt_comm(cmd)
             continue
 
+        if do_category and do_category not in unique.keys():
+            raise ValueError("Unknown category %s" % do_category)
+
         # Build up unique command line cases
         for category in unique.keys():
+            if do_category and category != do_category:
+                continue
             catdict = unique[category]
-            category_args = catdict["%s_args" % category]
+            category_args = catdict["args"]
 
             for optstr in catdict["valid"]:
-                cmdstr = "./%s %s %s %s" % (app, global_args,
+                cmdstr = "./%s %s %s %s" % (app, globalargs,
                                             category_args, optstr)
                 assertPass(cmdstr)
 
             for optstr in catdict["invalid"]:
-                cmdstr = "./%s %s %s %s" % (app, global_args,
+                cmdstr = "./%s %s %s %s" % (app, globalargs,
                                             category_args, optstr)
                 assertFail(cmdstr)
 
+            for optstr, filename in catdict.get("compare") or []:
+                filename = "%s/%s.xml" % (compare_xmldir, filename)
+                cmdstr = "./%s %s %s %s" % (app, globalargs,
+                                            category_args, optstr)
+                cmdstr = cmdstr % test_files
+
+                # Strip --debug to get reasonable output
+                cmdstr = cmdstr.replace("--debug ", "").replace("-d ", "")
+                if app == "virt-install":
+                    if (not cmdstr.count("--print-xml") and
+                        not cmdstr.count("--print-step") and
+                        not cmdstr.count("--quiet")):
+                        cmdstr += " --print-step all"
+
+                elif app == "virt-image":
+                    if not cmdstr.count("--print"):
+                        cmdstr += " --print"
+
+                elif app == "virt-clone":
+                    if not cmdstr.count("--print-xml"):
+                        cmdstr += " --print-xml"
+
+                if not cmdstr.count(fakeuri):
+                    cmdstr += " --connect %s" % fakeuri
+
+                check_fail = filename.endswith("fail.xml")
+
+                try:
+                    if check_fail:
+                        ignore, output = assertFail(cmdstr)
+                    else:
+                        ignore, output = assertPass(cmdstr)
+
+                    # Uncomment to generate new test files
+                    if not os.path.exists(filename):
+                        file(filename, "w").write(output)
+
+                    utils.diff_compare(output, filename)
+                except:
+                    print cmdstr
+                    raise
+
 def main():
     # CLI Args
     global debug
     global testprompt
 
     do_app = None
+    do_category = None
 
     if len(sys.argv) > 1:
         for i in range(1, len(sys.argv)):
@@ -827,7 +1060,9 @@ def main():
             elif sys.argv[i].count("prompt"):
                 testprompt = True
             elif sys.argv[i].count("--app"):
-                do_app = sys.argv[i+1]
+                do_app = sys.argv[i + 1]
+            elif sys.argv[i].count("--category"):
+                do_category = sys.argv[i + 1]
 
     # Setup needed files
     for i in exist_files:
@@ -845,10 +1080,12 @@ def main():
     os.system("chmod 555 %s" % ro_dir)
 
     try:
-        run_tests(do_app)
+        run_tests(do_app, do_category)
     finally:
         cleanup()
 
+    print "\nAll tests completed successfully."
+
 def cleanup():
     # Cleanup files
     for i in clean_files:
