@@ -2,6 +2,9 @@
 
 FILES="setup.py tests/ virt-install virt-image virt-clone virt-convert virtinst/ virtconv virtconv/parsers/*.py"
 
+# Don't print pylint config warning
+NO_PYL_CONFIG=".*No config file found.*"
+
 # Exceptions: deliberately ignore these regex
 
 # False positive: using the private excepthook is needed for custom exception
@@ -39,9 +42,13 @@ UCRED="import 'ucred'"
 SELINUX="import 'selinux'"
 COVERAGE="import 'coverage'"
 OLDSELINUX="'selinux' has no "
+HASHLIB_CONFUSION="Module 'hashlib' has no 'sha.*"
 
 # Public api error
-VD_MISMATCHED_ARGS="VirtualDisk.get_xml_config.*Arguments number differs"
+VD_MISMATCHED_ARGS="get_xml_config.*Arguments number differs"
+
+# Pylint getting confused with XMLBuilder complication
+XMLBUILDER_CONFUSION="xml_property.*protected member _xml.*"
 
 # urltest needs access to protected members for testing purposes
 URLTEST_ACCESS="TestURLFetch.*Access to a protected member"
@@ -67,7 +74,15 @@ ACCESS__CONN="Access to a protected member _conn"
 
 # There isn't a clean API way to access this functions from the API, but
 # they provide info that is needed. These need need to be fixed.
-PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_cleanup_install)|'virtinst.FullVirtGuest' has no '_OS_TYPES'|_install_bootconfig"
+PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_cleanup_install|_install_bootconfig|_channels|_get_caps|_open_test_uri)|'virtinst.FullVirtGuest' has no '_OS_TYPES'"
+
+
+# This doesn't belong here, but makes sure I don't break the test suite :)
+if grep -qIR crobinso tests --exclude pylint\* ; then
+    echo "Test suite borked:"
+    grep -IR crobinso tests --exclude pylint\*
+    exit 1
+fi
 
 DMSG=""
 addmsg() {
@@ -80,7 +95,7 @@ addchecker() {
 }
 
 addmsg_support() {
-    out=`pylint --list-msgs`
+    out=`pylint --list-msgs 2>&1`
     if `echo $out | grep -q $1` ; then
         addmsg "$1"
     fi
@@ -91,7 +106,7 @@ addmsg "C0103"  # C0103: Name doesn't match some style regex
 addmsg "C0111"  # C0111: No docstring
 addmsg "C0301"  # C0301: Line too long
 addmsg "C0302"  # C0302: Too many lines in module
-addmsg "W0105"  # W0105: String statement has no effect
+addmsg "W0105"  # W0105: String statement has no effect (annoying for docs)
 addmsg "W0141"  # W0141: Complaining about 'map' and 'filter'
 addmsg "W0142"  # W0142: Use of * or **
 addmsg "W0603"  # W0603: Using the global statement
@@ -103,9 +118,6 @@ addmsg "R0201"  # R0201: Method could be a function
 # Possibly useful at some point
 addmsg "W0403"  # W0403: Relative imports
 addmsg "W0511"  # W0511: FIXME and XXX: messages (useful in the future)
-addmsg "C0322"  # C0322: *Operator not preceded by a space*
-addmsg "C0323"  # C0323: *Operator not followed by a space*
-addmsg "C0324"  # C0324: *Comma not followed by a space*
 addmsg "R0401"  # R0401: Cyclic imports
 
 # Not supported in many pylint versions
@@ -124,9 +136,10 @@ pylint --ignore=coverage.py, $FILES \
   --reports=n \
   --output-format=colorized \
   --dummy-variables-rgx="dummy|ignore*|.*ignore" \
-  --disable-msg=${DMSG} \
-  --disable-checker=${DCHECKERS} 2>&1 | \
+  --disable=${DMSG} \
+  --disable=${DCHECKERS} 2>&1 | \
   egrep -ve "$EXCEPTHOOK" \
+        -ve "$NO_PYL_CONFIG" \
         -ve "$BTYPE_TYPE" \
         -ve "$BTYPE_FILE" \
         -ve "$BTYPE_STR" \
@@ -135,6 +148,7 @@ pylint --ignore=coverage.py, $FILES \
         -ve "$SELINUX" \
         -ve "$COVERAGE" \
         -ve "$OLDSELINUX" \
+        -ve "$HASHLIB_CONFUSION" \
         -ve "$USE_OF__EXIT" \
         -ve "$UNDEF_GETTEXT" \
         -ve "$VD_MISMATCHED_ARGS" \
@@ -144,6 +158,7 @@ pylint --ignore=coverage.py, $FILES \
         -ve "$TEST_HACKS" \
         -ve "$PROT_MEM_BUGS" \
         -ve "$CHAR_SUBCLASS" \
+        -ve "$XMLBUILDER_CONFUSION" \
         -ve "$OUTSIDE_INIT" | \
   $AWK '\
 # Strip out any "*** Module name" lines if we dont list any errors for them
@@ -164,3 +179,29 @@ BEGIN { found=0; cur_line="" }
         print $0
     }
 }'
+
+################
+# pep8 section #
+################
+
+SKIP_PEP8=""
+skip_pep8() {
+    if [ ! -z ${SKIP_PEP8} ] ; then
+        SKIP_PEP8="${SKIP_PEP8},"
+    fi
+    SKIP_PEP8="${SKIP_PEP8}$1"
+}
+
+skip_pep8 "E201"            # Spaces after [, before ]
+skip_pep8 "E203"            # Space before : in dictionary defs
+skip_pep8 "E221"            # Multiple spaces before operator (warns
+                            # about column aligning assigments
+skip_pep8 "E241"            # Space after , column alignment nono
+skip_pep8 "E261"            # 2 spaces before inline comment?
+skip_pep8 "E301"            # 1 blank line between methods
+skip_pep8 "E302"            # 2 blank lines between function defs
+skip_pep8 "E303"            # Too many blank lines
+skip_pep8 "E501"            # Line too long
+
+echo "Running pep8"
+pep8 -r --ignore $SKIP_PEP8 $FILES
