@@ -29,9 +29,9 @@ from virtinst import VirtualVideoDevice
 from virtinst import VirtualController
 from virtinst import VirtualWatchdog
 from virtinst import VirtualInputDevice
-import tests
+import utils
 
-conn = tests.open_testdriver()
+conn = utils.open_testdriver()
 scratch = os.path.join(os.getcwd(), "tests", "testscratchdir")
 
 def get_basic_paravirt_guest(testconn=conn, installer=None):
@@ -40,7 +40,7 @@ def get_basic_paravirt_guest(testconn=conn, installer=None):
     g.memory = int(200)
     g.maxmemory = int(400)
     g.uuid = "12345678-1234-1234-1234-123456789012"
-    g.boot = ["/boot/vmlinuz","/boot/initrd"]
+    g.boot = ["/boot/vmlinuz", "/boot/initrd"]
     g.graphics = (True, "vnc", None, "ja")
     g.vcpus = 5
 
@@ -68,8 +68,8 @@ def get_basic_fullyvirt_guest(typ="xen", testconn=conn, installer=None):
     g.installer._scratchdir = scratch
     return g
 
-def make_import_installer():
-    inst = virtinst.ImportInstaller(type="xen", os_type="hvm", conn=conn)
+def make_import_installer(os_type="hvm"):
+    inst = virtinst.ImportInstaller(type="xen", os_type=os_type, conn=conn)
     return inst
 
 def make_distro_installer(location="/default-pool/default-vol", gtype="xen"):
@@ -98,18 +98,18 @@ def build_win_kvm(path=None):
 
     return g
 
-def get_floppy(path = None):
+def get_floppy(path=None):
     if not path:
         path = "/default-pool/testvol1.img"
     return VirtualDisk(path, conn=conn, device=VirtualDisk.DEVICE_FLOPPY)
 
-def get_filedisk(path = None):
+def get_filedisk(path=None):
     if not path:
         path = "/tmp/test.img"
     return VirtualDisk(path, size=.0001, conn=conn)
 
-def get_blkdisk():
-    return VirtualDisk("/dev/loop0", conn=conn)
+def get_blkdisk(path="/dev/loop0"):
+    return VirtualDisk(path, conn=conn)
 
 def get_virtual_network():
     dev = virtinst.VirtualNetworkInterface()
@@ -129,19 +129,6 @@ def build_xmlfile(filebase):
         return None
     return os.path.join("tests/xmlconfig-xml", filebase + ".xml")
 
-def sanitize_xml(xml):
-    # Libvirt throws errors since we are defining domain
-    # type='xen', when test driver can only handle type='test'
-    # Sanitize the XML so we can define
-    if not xml:
-        return xml
-
-    xml = xml.replace("<domain type='xen'>",
-                      "<domain type='test'>")
-    xml = xml.replace(">linux<", ">xen<")
-
-    return xml
-
 class TestXMLConfig(unittest.TestCase):
 
     def tearDown(self):
@@ -156,29 +143,11 @@ class TestXMLConfig(unittest.TestCase):
             actualXML = guest.get_config_xml(install=do_install,
                                              disk_boot=do_disk_boot)
 
-            tests.diff_compare(actualXML, filename)
-            self._testCreate(guest.conn, actualXML)
+            utils.diff_compare(actualXML, filename)
+            utils.test_create(guest.conn, actualXML)
         finally:
             guest._cleanup_install()
 
-    def _testCreate(self, testconn, xml):
-        xml = sanitize_xml(xml)
-
-        dom = testconn.defineXML(xml)
-        try:
-            dom.create()
-            dom.destroy()
-            dom.undefine()
-        except:
-            try:
-                dom.destroy()
-            except:
-                pass
-            try:
-                dom.undefine()
-            except:
-                pass
-
     def _testInstall(self, guest,
                      instxml=None, bootxml=None, contxml=None):
         instname = build_xmlfile(instxml)
@@ -193,7 +162,7 @@ class TestXMLConfig(unittest.TestCase):
         old_getxml = guest.get_config_xml
         def new_getxml(install=True, disk_boot=False):
             xml = old_getxml(install, disk_boot)
-            return sanitize_xml(xml)
+            return utils.sanitize_xml_for_define(xml)
         guest.get_config_xml = new_getxml
 
         try:
@@ -211,11 +180,11 @@ class TestXMLConfig(unittest.TestCase):
             xmlcont = guest.get_config_xml(True, True)
 
             if instname:
-                tests.diff_compare(xmlinst, instname)
+                utils.diff_compare(xmlinst, instname)
             if contname:
-                tests.diff_compare(xmlcont, contname)
+                utils.diff_compare(xmlcont, contname)
             if bootname:
-                tests.diff_compare(xmlboot, bootname)
+                utils.diff_compare(xmlboot, bootname)
 
             if guest.get_continue_inst():
                 guest.continue_install(consolecb, meter, wait)
@@ -237,7 +206,9 @@ class TestXMLConfig(unittest.TestCase):
                                conn_version=None,
                                conn_uri=None,
                                libvirt_version=None):
-        testconn = guest.conn
+        testconn = guest
+        if isinstance(guest, virtinst.Guest):
+            testconn = guest.conn
 
         def set_func(newfunc, funcname, obj, force=False):
             if newfunc or force:
@@ -275,9 +246,25 @@ class TestXMLConfig(unittest.TestCase):
 
     def testBootParavirtDiskFile(self):
         g = get_basic_paravirt_guest()
-        g.disks.append(get_filedisk())
+        g.disks.append(get_filedisk("/tmp/somerandomfilename.img"))
         self._compare(g, "boot-paravirt-disk-file", False)
 
+        # Just cram some post_install_checks in here
+        try:
+            g.post_install_check()
+            raise AssertionError("Expected OSError, none caught.")
+        except OSError:
+            pass
+
+        g.disks[0].path = "virt-install"
+        self.assertEquals(g.post_install_check(), False)
+
+        g.disks[0].driver_type = "raw"
+        self.assertEquals(g.post_install_check(), False)
+
+        g.disks[0].driver_type = "foobar"
+        self.assertEquals(g.post_install_check(), True)
+
     def testBootParavirtDiskFileBlktapCapable(self):
         oldblktap = virtinst._util.is_blktap_capable
         try:
@@ -425,6 +412,7 @@ class TestXMLConfig(unittest.TestCase):
         g.installer.bootconfig.bootorder = [
             g.installer.bootconfig.BOOT_DEVICE_NETWORK]
         g.installer.bootconfig.enable_bootmenu = True
+        g.seclabel.model = "default"
 
         self._compare(g, "boot-fullyvirt-pxe-always", False)
 
@@ -520,7 +508,7 @@ class TestXMLConfig(unittest.TestCase):
         self._compare(g, "install-fullyvirt-import-multiboot", False)
 
     def testInstallPVImport(self):
-        i = make_import_installer()
+        i = make_import_installer("xen")
         g = get_basic_paravirt_guest(installer=i)
 
         g.disks.append(get_filedisk())
@@ -534,6 +522,7 @@ class TestXMLConfig(unittest.TestCase):
 
         g = get_basic_fullyvirt_guest()
         g.disks.append(get_filedisk())
+        g.disks.append(get_blkdisk("/iscsi-pool/diskvol1"))
         fargs = (g, "misc-qemu-driver-type", True)
         self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
@@ -550,6 +539,7 @@ class TestXMLConfig(unittest.TestCase):
 
     def testXMLEscaping(self):
         g = get_basic_fullyvirt_guest()
+        g.description = "foooo barrrr \n baz && snarf. '' \"\" @@$\n"
         g.disks.append(get_filedisk("/tmp/ISO&'&s"))
         self._compare(g, "misc-xml-escaping", True)
 
@@ -625,6 +615,26 @@ class TestXMLConfig(unittest.TestCase):
                                     conn_version=conn_nosupport_ac97,
                                     conn_uri=qemu_uri)
 
+    def testKVMKeymap(self):
+        def conn_nosupport_autokeymap():
+            return 10000
+        def conn_support_autokeymap():
+            return 11000
+
+        def test1():
+            g = virtinst.VirtualGraphics(conn=conn, type="vnc")
+            self.assertTrue(g.keymap != None)
+        self.conn_function_wrappers(conn, (), func=test1,
+                                    conn_uri=qemu_uri,
+                                    conn_version=conn_nosupport_autokeymap)
+
+        def test2():
+            g = virtinst.VirtualGraphics(conn=conn, type="vnc")
+            self.assertTrue(g.keymap == None)
+        self.conn_function_wrappers(conn, (), func=test2,
+                                    conn_uri=qemu_uri,
+                                    conn_version=conn_support_autokeymap)
+
 
     def testF11Qemu(self):
         i = make_distro_installer(gtype="qemu")
@@ -732,11 +742,16 @@ class TestXMLConfig(unittest.TestCase):
         net3.model = "e1000"
         net4 = VirtualNetworkInterface(bridge="foobr0",
                                        macaddr="22:22:22:22:22:22")
+        net4.target_dev = "foo1"
+        net5 = VirtualNetworkInterface(type="ethernet",
+                                       macaddr="00:11:00:22:00:33")
+        net5.source_dev = "testeth1"
 
         g.nics.append(net1)
         g.nics.append(net2)
         g.nics.append(net3)
         g.nics.append(net4)
+        g.nics.append(net5)
         self._compare(g, "boot-many-nics", False)
 
     def testManyHostdevs(self):
@@ -803,19 +818,32 @@ class TestXMLConfig(unittest.TestCase):
                                                   VirtualCharDevice.CHAR_PTY)
         dev6.target_type = dev5.CHAR_CONSOLE_TARGET_VIRTIO
 
+        dev7 = VirtualCharDevice.get_dev_instance(g.conn,
+                                                  VirtualCharDevice.DEV_CONSOLE,
+                                                  VirtualCharDevice.CHAR_PTY)
+
+        dev8 = VirtualCharDevice.get_dev_instance(g.conn,
+                                                  VirtualCharDevice.DEV_CHANNEL,
+                                                  VirtualCharDevice.CHAR_PTY)
+        dev8.target_type = dev5.CHAR_CHANNEL_TARGET_GUESTFWD
+        dev8.target_address = "1.2.3.4"
+        dev8.target_port = "4567"
+
         g.add_device(dev1)
         g.add_device(dev2)
         g.add_device(dev3)
         g.add_device(dev4)
         g.add_device(dev5)
         g.add_device(dev6)
+        g.add_device(dev7)
+        g.add_device(dev8)
         self._compare(g, "boot-many-chars", False)
 
     def testManyDevices(self):
         i = make_pxe_installer()
         g = get_basic_fullyvirt_guest(installer=i)
 
-        g.description = "foooo barrrr \n baz && snarf. '' \"\" @@$\n"
+        g.description = "foooo barrrr somedesc"
 
         # Hostdevs
         dev1 = VirtualHostDeviceUSB(g.conn)
@@ -875,23 +903,46 @@ class TestXMLConfig(unittest.TestCase):
         vdev2.heads = 3
 
         vdev3 = VirtualVideoDevice(g.conn)
+        vdev4 = VirtualVideoDevice(g.conn)
+        vdev4.model_type = "qxl"
+
         g.add_device(vdev1)
         g.add_device(vdev2)
         g.add_device(vdev3)
+        g.add_device(vdev4)
 
         wdev2 = VirtualWatchdog(g.conn)
         wdev2.model = "ib700"
         wdev2.action = "none"
         g.add_device(wdev2)
 
+        # Check keymap autoconfig
+        gdev1 = virtinst.VirtualGraphics(conn=g.conn, type="vnc")
+        self.assertTrue(gdev1.keymap != None)
+        gdev1.keymap = "en-us"
+
+        # Check keymap None
+        gdev2 = virtinst.VirtualGraphics(conn=g.conn, type="vnc")
+        gdev2.keymap = None
+
+        gdev3 = virtinst.VirtualGraphics(conn=g.conn, type="sdl")
+        gdev4 = virtinst.VirtualGraphics(conn=g.conn, type="spice")
+
+        gdev5 = virtinst.VirtualGraphics(conn=g.conn, type="sdl")
+        gdev5.xauth = "fooxauth"
+        gdev5.display = "foodisplay"
+        g.add_device(gdev1)
+        g.add_device(gdev2)
+        g.add_device(gdev3)
+        g.add_device(gdev4)
+        g.add_device(gdev5)
+
         g.clock.offset = "localtime"
 
-        seclabel = virtinst.Seclabel(g.conn)
-        seclabel.type = seclabel.SECLABEL_TYPE_STATIC
-        seclabel.model = "selinux"
-        seclabel.label = "foolabel"
-        seclabel.imagelabel = "imagelabel"
-        g.seclabel = seclabel
+        g.seclabel.type = g.seclabel.SECLABEL_TYPE_STATIC
+        g.seclabel.model = "selinux"
+        g.seclabel.label = "foolabel"
+        g.seclabel.imagelabel = "imagelabel"
 
         self._compare(g, "boot-many-devices", False)
 
@@ -902,9 +953,45 @@ class TestXMLConfig(unittest.TestCase):
         # Cpuset
         cpustr = g.generate_cpuset(g.conn, g.memory)
         g.cpuset = cpustr
+        g.maxvcpus = 7
+
+        g.cpu.model = "footest"
+        g.cpu.vendor = "Intel"
+        g.cpu.match = "minimum"
+
+        g.cpu.threads = "2"
+        g.cpu.sockets = "4"
+        g.cpu.cores = "5"
+
+        g.cpu.add_feature("x2apic", "force")
+        g.cpu.add_feature("lahf_lm", "forbid")
 
         self._compare(g, "boot-cpuset", False)
 
+        # Test CPU topology determining
+        cpu = virtinst.CPU(g.conn)
+        cpu.sockets = "2"
+        cpu.set_topology_defaults(6)
+        self.assertEquals([cpu.sockets, cpu.cores, cpu.threads], [2, 3, 1])
+
+        cpu = virtinst.CPU(g.conn)
+        cpu.cores = "4"
+        cpu.set_topology_defaults(9)
+        self.assertEquals([cpu.sockets, cpu.cores, cpu.threads], [2, 4, 1])
+
+        cpu = virtinst.CPU(g.conn)
+        cpu.threads = "3"
+        cpu.set_topology_defaults(14)
+        self.assertEquals([cpu.sockets, cpu.cores, cpu.threads], [4, 1, 3])
+
+        cpu = virtinst.CPU(g.conn)
+        cpu.sockets = 5
+        cpu.cores = 2
+        self.assertEquals(cpu.vcpus_from_topology(), 10)
+
+        cpu = virtinst.CPU(g.conn)
+        self.assertEquals(cpu.vcpus_from_topology(), 1)
+
 
     #
     # Full Install tests: try to mimic virt-install as much as possible
