@@ -113,7 +113,7 @@ def export_os_params(vm):
 def export_disks(vm):
     """
     Export code for the disks.  Slightly tricky for two reasons.
-    
+
     We can't handle duplicate disks: some vmx files define SCSI/IDE devices
     that point to the same storage, and Xen isn't happy about that. We
     just ignore any entries that have duplicate paths.
@@ -228,17 +228,22 @@ class virtimage_parser(formats.parser):
 
         for d in boot.drives:
             disk = d.disk
+            format_mappings = {
+                ImageParser.Disk.FORMAT_RAW: diskcfg.DISK_FORMAT_RAW,
+                ImageParser.Disk.FORMAT_VMDK: diskcfg.DISK_FORMAT_VMDK,
+                ImageParser.Disk.FORMAT_QCOW: diskcfg.DISK_FORMAT_QCOW,
+                ImageParser.Disk.FORMAT_QCOW2: diskcfg.DISK_FORMAT_QCOW2,
+            }
+
             fmt = None
-            if disk.format == ImageParser.Disk.FORMAT_RAW:
-                fmt = diskcfg.DISK_FORMAT_RAW
-            elif disk.format == ImageParser.Disk.FORMAT_VMDK:
-                fmt = diskcfg.DISK_FORMAT_VMDK
+            if disk.format in format_mappings:
+                fmt = format_mappings[disk.format]
+            else:
+                raise ValueError(_("Unknown disk format '%s'") % disk.format)
 
-            if fmt is None:
-                raise ValueError(_("Unable to determine disk format"))
             devid = (bus, nr_disk)
-            vm.disks[devid] = diskcfg.disk(bus = bus,
-                type = diskcfg.DISK_TYPE_DISK)
+            vm.disks[devid] = diskcfg.disk(bus=bus,
+                type=diskcfg.DISK_TYPE_DISK)
             vm.disks[devid].format = fmt
             vm.disks[devid].path = disk.file
             nr_disk = nr_disk + 1
@@ -246,7 +251,8 @@ class virtimage_parser(formats.parser):
         nics = domain.interface
         nic_idx = 0
         while nic_idx in range(0, nics):
-            vm.netdevs[nic_idx] = netdevcfg.netdev(type = netdevcfg.NETDEV_TYPE_UNKNOWN)
+            vm.netdevs[nic_idx] = netdevcfg.netdev(
+                                    type=netdevcfg.NETDEV_TYPE_UNKNOWN)
             nic_idx = nic_idx + 1
             """  Eventually need to add support for mac addresses if given"""
         vm.validate()
