@@ -19,6 +19,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
+import re
+
 from virtinst import _virtinst as _
 import _util
 
@@ -30,19 +32,127 @@ class CapabilitiesParserException(Exception):
 FEATURE_ON      = 0x01
 FEATURE_OFF     = 0x02
 
+class CPUValuesModel(object):
+    """
+    Single <model> definition from cpu_map
+    """
+    def __init__(self, node):
+        self.model = node.prop("name")
+        self.features = []
+        self.parent = None
+        self.vendor = None
+
+        self._parseXML(node)
+
+    def _parseXML(self, node):
+        child = node.children
+        while child:
+            if child.name == "model":
+                self.parent = child.prop("name")
+            if child.name == "vendor":
+                self.vendor = child.prop("name")
+            if child.name == "feature":
+                self.features.append(child.prop("name"))
+
+            child = child.next
+
+        self.features.sort()
+
+    def inheritParent(self, parentcpu):
+        self.vendor = parentcpu.vendor or self.vendor
+        self.features += parentcpu.features
+        self.features.sort()
+
+
+class CPUValuesArch(object):
+    """
+    Single <arch> instance of valid CPUs
+    """
+    def __init__(self, arch, node=None):
+        self.arch = arch
+        self.vendors = []
+        self.cpus = []
+        self.features = []
+
+        if node:
+            self._parseXML(node)
+
+    def _parseXML(self, node):
+        child = node.children
+        while child:
+            if child.name == "vendor":
+                self.vendors.append(child.prop("name"))
+            if child.name == "feature":
+                self.features.append(child.prop("name"))
+            if child.name == "model":
+                newcpu = CPUValuesModel(child)
+                if newcpu.parent:
+                    for chkcpu in self.cpus:
+                        if chkcpu.model == newcpu.parent:
+                            newcpu.inheritParent(chkcpu)
+                self.cpus.append(newcpu)
+
+            child = child.next
+
+        self.vendors.sort()
+        self.features.sort()
+
+    def get_cpu(self, model):
+        for c in self.cpus:
+            if c.model == model:
+                return c
+        raise ValueError(_("Unknown CPU model '%s'") % model)
+
+class CPUValues(object):
+    """
+    Lists valid values for domain <cpu> parameters, parsed from libvirt's
+    local cpu_map.xml
+    """
+    def __init__(self, cpu_filename=None):
+        self.archmap = {}
+        if not cpu_filename:
+            cpu_filename = "/usr/share/libvirt/cpu_map.xml"
+        xml = file(cpu_filename).read()
+
+        _util.parse_node_helper(xml, "cpus",
+                                self._parseXML,
+                                CapabilitiesParserException)
+
+    def _parseXML(self, node):
+        child = node.children
+        while child:
+            if child.name == "arch":
+                arch = child.prop("name")
+                self.archmap[arch] = CPUValuesArch(arch, child)
+
+            child = child.next
+
+    def get_arch(self, arch):
+        if re.match(r'i[4-9]86', arch):
+            arch = "x86"
+        elif arch == "x86_64":
+            arch = "x86"
+
+        cpumap = self.archmap.get(arch)
+        if not cpumap:
+            cpumap = CPUValuesArch(arch)
+            self.archmap[arch] = cpumap
+
+        return cpumap
+
 class Features(object):
     """Represent a set of features. For each feature, store a bit mask of
        FEATURE_ON and FEATURE_OFF to indicate whether the feature can
        be turned on or off. For features for which toggling doesn't make sense
        (e.g., 'vmx') store FEATURE_ON when the feature is present."""
 
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.features = {}
         if node is not None:
             self.parseXML(node)
 
     def __getitem__(self, feature):
-        if self.features.has_key(feature):
+        if feature in self.features:
             return self.features[feature]
         return 0
 
@@ -64,7 +174,7 @@ class Features(object):
                 feature_list.append(n.content)
 
         for feature in feature_list:
-            if not d.has_key(feature):
+            if feature not in d:
                 d[feature] = 0
 
             self._extractFeature(feature, d, n)
@@ -75,7 +185,7 @@ class Features(object):
         raise NotImplementedError("Abstract base class")
 
 class CapabilityFeatures(Features):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         Features.__init__(self, node)
 
     def _extractFeature(self, feature, d, n):
@@ -91,7 +201,7 @@ class CapabilityFeatures(Features):
             else:
                 raise CapabilitiesParserException("Feature %s: value of default must be 'on' or 'off', but is '%s'" % (feature, default))
             if toggle == "yes":
-                d[feature] |= d[feature] ^ (FEATURE_ON|FEATURE_OFF)
+                d[feature] |= d[feature] ^ (FEATURE_ON | FEATURE_OFF)
         else:
             # Format for old HOST features, on OLD old guest features
             # back compat is just <$featurename>, like <svm/>
@@ -105,6 +215,7 @@ class CPU(object):
         # e.g. "i686" or "x86_64"
         self.arch = None
         self.model = None
+        self.vendor = None
         self.sockets = 1
         self.cores = 1
         self.threads = 1
@@ -133,6 +244,8 @@ class CPU(object):
                 self.arch = child.content
             elif child.name == "model":
                 self.model = child.content
+            elif child.name == "vendor":
+                self.vendor = child.content
             elif child.name == "topology":
                 self.sockets = xpathString(child, "@sockets") or 1
                 self.cores = xpathString(child, "@cores") or 1
@@ -144,7 +257,7 @@ class CPU(object):
             child = child.next
 
 class Host(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.cpu = CPU()
         self.topology = None
         self.secmodel = None
@@ -181,7 +294,7 @@ class Host(object):
 
 
 class Guest(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         # e.g. "xen" or "hvm"
         self.os_type = None
         # e.g. "i686" or "x86_64"
@@ -210,6 +323,10 @@ class Guest(object):
                 while n:
                     if n.name == "machine":
                         machines.append(n.content)
+
+                        canon = n.prop("canonical")
+                        if canon:
+                            machines.append(canon)
                     elif n.name == "emulator":
                         emulator = n.content
                     elif n.name == "loader":
@@ -219,38 +336,64 @@ class Guest(object):
                 n = child.children
                 while n:
                     if n.name == "domain":
-                        self.domains.append(Domain(n.prop("type"), emulator, loader, machines, n))
+                        self.domains.append(Domain(n.prop("type"),
+                                            emulator, loader, machines, n))
                     n = n.next
 
             child = child.next
 
-
-    def bestDomainType(self, accelerated=None):
-        if len(self.domains) == 0:
-            raise CapabilitiesParserException(_("No domains available for "
-                                                "virt type '%(type)s', arch "
-                                                "%(arch)s.") % \
-                                                {'type': self.os_type,
-                                                 'arch': self.arch})
+    def _favoredDomain(self, accelerated, domains):
+        """
+        Return the recommended domain for use if the user does not explicitly
+        request one.
+        """
         if accelerated is None:
             # Picking last in list so we favour KVM/KQEMU over QEMU
-            return self.domains[-1]
-        else:
-            priority = ["kvm", "xen", "kqemu", "qemu"]
-            if not accelerated:
-                priority.reverse()
+            return domains[-1]
 
-            for t in priority:
-                for d in self.domains:
-                    if d.hypervisor_type == t:
-                        return d
+        priority = ["kvm", "xen", "kqemu", "qemu"]
+        if not accelerated:
+            priority.reverse()
 
-            # Fallback, just return last item in list
-            return self.domains[-1]
+        for t in priority:
+            for d in domains:
+                if d.hypervisor_type == t:
+                    return d
+
+        # Fallback, just return last item in list
+        return domains[-1]
+
+    def bestDomainType(self, accelerated=None, dtype=None, machine=None):
+        domains = []
+        for d in self.domains:
+            if dtype and d.hypervisor_type != dtype.lower():
+                continue
+            if machine and machine not in d.machines:
+                continue
+            domains.append(d)
+
+        if len(domains) == 0:
+            domainerr = ""
+            machineerr = ""
+            if dtype:
+                domainerr = _(", domain type '%s'") % dtype
+            if machine:
+                machineerr = _(", machine type '%s'") % machine
+
+            error = (_("No domains available for virt type '%(type)s', "
+                      "arch '%(arch)s'") %
+                      {'type': self.os_type, 'arch': self.arch})
+            error += domainerr
+            error += machineerr
+            raise CapabilitiesParserException(error)
+
+        return self._favoredDomain(accelerated, domains)
 
 
 class Domain(object):
-    def __init__(self, hypervisor_type, emulator = None, loader = None, machines = None, node = None):
+    def __init__(self, hypervisor_type,
+                 emulator=None, loader=None,
+                 machines=None, node=None):
         self.hypervisor_type = hypervisor_type
         self.emulator = emulator
         self.loader = loader
@@ -268,13 +411,21 @@ class Domain(object):
                 self.emulator = child.content
             elif child.name == "machine":
                 machines.append(child.content)
+
+                canon = child.prop("canonical")
+                if canon:
+                    machines.append(canon)
+                machines.append(child.content)
             child = child.next
 
         if len(machines) > 0:
             self.machines = machines
 
+    def is_accelerated(self):
+        return self.hypervisor_type in ["kvm", "kqemu"]
+
 class Topology(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.cells = []
 
         if not node is None:
@@ -288,7 +439,7 @@ class Topology(object):
                     self.cells.append(TopologyCell(cell))
 
 class TopologyCell(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.id = None
         self.cpus = []
 
@@ -304,7 +455,7 @@ class TopologyCell(object):
                     self.cpus.append(TopologyCPU(cpu))
 
 class TopologyCPU(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.id = None
 
         if not node is None:
@@ -315,7 +466,7 @@ class TopologyCPU(object):
 
 
 class SecurityModel(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.model = None
         self.doi = None
 
@@ -330,10 +481,11 @@ class SecurityModel(object):
                 self.doi = child.content
 
 class Capabilities(object):
-    def __init__(self, node = None):
+    def __init__(self, node=None):
         self.host = None
         self.guests = []
         self._topology = None
+        self._cpu_values = None
 
         if not node is None:
             self.parseXML(node)
@@ -442,7 +594,13 @@ class Capabilities(object):
 
         return True
 
-    def guestForOSType(self, type = None, arch = None):
+    def support_pae(self):
+        for g in self.guests:
+            if "pae" in g.features.names():
+                return True
+        return False
+
+    def guestForOSType(self, type=None, arch=None):
         if self.host is None:
             return None
 
@@ -450,6 +608,7 @@ class Capabilities(object):
             archs = [self.host.arch, None]
         else:
             archs = [arch]
+
         for a in archs:
             for g in self.guests:
                 if (type is None or g.os_type == type) and \
@@ -497,13 +656,19 @@ class Capabilities(object):
         if self.host.topology is None:
             self.host.topology = self._topology
 
+    def get_cpu_values(self, arch):
+        if not self._cpu_values:
+            self._cpu_values = CPUValues()
+
+        return self._cpu_values.get_arch(arch)
+
 def parse(xml):
     return _util.parse_node_helper(xml, "capabilities",
                                    Capabilities,
                                    CapabilitiesParserException)
 
 def guest_lookup(conn, caps=None, os_type=None, arch=None, type=None,
-                 accelerated=False):
+                 accelerated=False, machine=None):
     """
     Simple virtualization availability lookup
 
@@ -533,6 +698,8 @@ def guest_lookup(conn, caps=None, os_type=None, arch=None, type=None,
     @param accelerated: Whether to look for accelerated domain if none is
                         specifically requested
     @type accelerated: C{bool}
+    @param machine: Optional machine type to emulate
+    @type machine: C{str}
 
     @returns: A (Capabilities Guest, Capabilities Domain) tuple
     """
@@ -553,26 +720,24 @@ def guest_lookup(conn, caps=None, os_type=None, arch=None, type=None,
         raise ValueError(_("Host does not support %(virttype)s %(arch)s") %
                            {'virttype' : osstr, 'arch' : archstr})
 
-    domain = None
-    if type:
-        for d in guest.domains:
-            if d.hypervisor_type == type.lower():
-                domain = d
-                break
-    else:
-        domain = guest.bestDomainType(accelerated=accelerated)
+    domain = guest.bestDomainType(accelerated=accelerated,
+                                  dtype=type,
+                                  machine=machine)
 
     if domain == None:
-        raise ValueError(_("Host does not support domain type "
-                           "'%(domain)s' for virtualization type "
+        machinestr = "with machine '%s'" % machine
+        if not machine:
+            machinestr = ""
+        raise ValueError(_("Host does not support domain type %(domain)s"
+                           "%(machine)s for virtualization type "
                            "'%(virttype)s' arch '%(arch)s'") %
                            {'domain': type, 'virttype': guest.os_type,
-                            'arch': guest.arch})
+                            'arch': guest.arch, 'machine': machinestr})
 
     return (guest, domain)
 
 
-def xpathString(node, path, default = None):
+def xpathString(node, path, default=None):
     result = node.xpathEval("string(%s)" % path)
     if len(result) == 0:
         result = default
