@@ -34,6 +34,7 @@ General workflow for cloning:
 
 import logging
 import re
+import os
 
 import libxml2
 import urlgrabber.progress as progress
@@ -55,8 +56,10 @@ def _listify(val):
     else:
         return False, [val]
 
-def generate_clone_disk_path(origpath, design):
-    basename = origpath
+def generate_clone_disk_path(origpath, design, newname=None):
+    origname = design.original_guest
+    newname = newname or design.clone_name
+    path = origpath
     suffix = ""
 
     # Try to split the suffix off the existing disk name. Ex.
@@ -65,14 +68,23 @@ def generate_clone_disk_path(origpath, design):
     # If the suffix is greater than 7 characters, assume it isn't
     # a file extension and is part of the disk name, at which point
     # just stick '-clone' on the end.
-    if basename.count(".") and len(basename.rsplit(".", 1)[1]) <= 7:
-        basename, suffix = basename.rsplit(".", 1)
+    if origpath.count(".") and len(origpath.rsplit(".", 1)[1]) <= 7:
+        path, suffix = origpath.rsplit(".", 1)
         suffix = "." + suffix
 
-    return _util.generate_name(basename + "-clone",
-                               lambda p: VirtualDisk.path_exists(design.original_conn, p),
-                               suffix,
-                               lib_collision=False)
+    dirname = os.path.dirname(path)
+    basename = os.path.basename(path)
+
+    clonebase = basename + "-clone"
+    if origname and basename == origname:
+        clonebase = newname
+
+    clonebase = os.path.join(dirname, clonebase)
+    return _util.generate_name(
+                    clonebase,
+                    lambda p: VirtualDisk.path_exists(design.original_conn, p),
+                    suffix,
+                    lib_collision=False)
 
 def generate_clone_name(design):
     # If the orig name is "foo-clone", we don't want the clone to be
@@ -119,7 +131,7 @@ class CloneDesign(object):
         self._clone_name         = None
         self._clone_devices      = []
         self._clone_virtual_disks = []
-        self._clone_bs           = 1024*1024*10
+        self._clone_bs           = 1024 * 1024 * 10
         self._clone_mac          = []
         self._clone_uuid         = None
         self._clone_sparse       = True
@@ -128,6 +140,7 @@ class CloneDesign(object):
         self._force_target       = []
         self._skip_target        = []
         self._preserve           = True
+        self._clone_running      = False
 
         # Default clone policy for back compat: don't clone readonly,
         # shareable, or empty disks
@@ -174,7 +187,7 @@ class CloneDesign(object):
         try:
             self._valid_guest.set_name(name)
         except ValueError, e:
-            raise ValueError, _("Invalid name for new guest: %s") % (str(e),)
+            raise ValueError(_("Invalid name for new guest: %s") % e)
 
         # Make sure new VM name isn't taken.
         try:
@@ -192,7 +205,7 @@ class CloneDesign(object):
         try:
             self._valid_guest.set_uuid(uuid)
         except ValueError, e:
-            raise ValueError, _("Invalid uuid for new guest: %s") % (str(e),)
+            raise ValueError(_("Invalid uuid for new guest: %s") % e)
 
         if _util.vm_uuid_collision(self._hyper_conn, uuid):
             raise ValueError(_("UUID '%s' is in use by another guest.") %
@@ -368,6 +381,15 @@ class CloneDesign(object):
                             doc="List of policy rules for determining which "
                                 "vm disks to clone. See CLONE_POLICY_*")
 
+    def get_clone_running(self):
+        return self._clone_running
+    def set_clone_running(self, val):
+        self._clone_running = bool(val)
+    clone_running = property(get_clone_running, set_clone_running,
+                             doc="Allow cloning a running VM. If enabled, "
+                                 "domain state is not checked before "
+                                 "cloning.")
+
     # Functional methods
 
     def setup_original(self):
@@ -390,13 +412,14 @@ class CloneDesign(object):
         logging.debug("Original sizes: %s" % (self.original_devices_size))
 
         # If domain has devices to clone, it must be 'off' or 'paused'
-        if self._original_dom and len(self.original_devices) != 0:
+        if (not self.clone_running and
+            (self._original_dom and len(self.original_devices) != 0)):
             status = self._original_dom.info()[0]
 
             if status not in [libvirt.VIR_DOMAIN_SHUTOFF,
                               libvirt.VIR_DOMAIN_PAUSED]:
-                raise RuntimeError, _("Domain with devices to clone must be "
-                                      "paused or shutoff.")
+                raise RuntimeError(_("Domain with devices to clone must be "
+                                     "paused or shutoff."))
 
 
     def setup_clone(self):
@@ -426,7 +449,7 @@ class CloneDesign(object):
 
         # changing mac
         count = ctx.xpathEval("count(/domain/devices/interface/mac)")
-        for i in range(1, int(count+1)):
+        for i in range(1, int(count + 1)):
             base_xpath = "/domain/devices/interface[%d]" % i
             base_node = ctx.xpathEval(base_xpath)[0]
             node = ctx.xpathEval(base_xpath + "/mac/@address")
@@ -439,7 +462,7 @@ class CloneDesign(object):
 
             mac = None
             try:
-                mac = self._clone_mac[i-1]
+                mac = self._clone_mac[i - 1]
             except Exception:
                 while 1:
                     mac = _util.randomMAC(typ)
@@ -452,7 +475,7 @@ class CloneDesign(object):
             node.setContent(mac)
 
         if len(self.clone_virtual_disks) < len(self.original_virtual_disks):
-            raise ValueError(_("More disks to clone that new paths specified. "
+            raise ValueError(_("More disks to clone than new paths specified. "
                                "(%(passed)d specified, %(need)d needed") %
                                {"passed" : len(self.clone_virtual_disks),
                                 "need"   : len(self.original_virtual_disks) })
@@ -561,7 +584,7 @@ class CloneDesign(object):
         lst     = []
 
         count = _util.get_xml_path(xml, "count(/domain/devices/disk)")
-        for i in range(1, int(count+1)):
+        for i in range(1, int(count + 1)):
             # Check if the disk needs cloning
             (path, target) = self._do_we_clone_device(xml, i)
             if target == None:
@@ -680,4 +703,3 @@ def _do_duplicate(design, meter):
 
         # VirtualDisk.setup handles everything
         dst_dev.setup(meter)
-
