@@ -65,10 +65,13 @@ def _sanitize_url(url):
     return url
 
 class DistroInstaller(Installer.Installer):
-    def __init__(self, type = "xen", location = None, boot = None,
-                 extraargs = None, os_type = None, conn = None):
+    def __init__(self, type="xen", location=None, boot=None,
+                 extraargs=None, os_type=None,
+                 conn=None, caps=None):
         Installer.Installer.__init__(self, type, location, boot, extraargs,
-                                 os_type, conn=conn)
+                                     os_type, conn=conn, caps=caps)
+
+        self._livecd = False
 
         # True == location is a filesystem path
         # False == location is a url
@@ -76,6 +79,12 @@ class DistroInstaller(Installer.Installer):
 
     # DistroInstaller specific methods/overwrites
 
+    def _get_livecd(self):
+        return self._livecd
+    def _set_livecd(self, val):
+        self._livecd = bool(val)
+    livecd = property(_get_livecd, _set_livecd)
+
     def get_location(self):
         return self._location
     def set_location(self, val):
@@ -158,6 +167,7 @@ class DistroInstaller(Installer.Installer):
     # Private helper methods
 
     def _prepare_cdrom(self, guest, meter):
+        transient = not self.livecd
         if not self._location_is_path:
             # Xen needs a boot.iso if its a http://, ftp://, or nfs: url
             (store_ignore, os_type_ignore, os_variant_ignore, media) = \
@@ -166,6 +176,7 @@ class DistroInstaller(Installer.Installer):
             cdrom = media
 
             self._tmpfiles.append(cdrom)
+            transient = True
         else:
             cdrom = self.location
 
@@ -173,47 +184,36 @@ class DistroInstaller(Installer.Installer):
                            conn=guest.conn,
                            device=VirtualDisk.DEVICE_CDROM,
                            readOnly=True,
-                           transient=True)
+                           transient=transient)
         self.install_devices.append(disk)
 
     def _perform_initrd_injections(self):
         """
         Insert files into the root directory of the initial ram disk
         """
-        logging.debug("Unpacking initrd.")
         initrd = self._install_bootconfig.initrd
         tempdir = tempfile.mkdtemp(dir=self.scratchdir)
         os.chmod(tempdir, 0775)
 
-        gzip_proc = subprocess.Popen(['gzip', '-dc', initrd],
-                                     stdout=subprocess.PIPE, stderr=sys.stderr)
-        cpio_proc = subprocess.Popen(['cpio', '-i', '-d', '--quiet'],
-                                     stdin=gzip_proc.stdout,
-                                     stderr=sys.stderr, cwd=tempdir)
-        cpio_proc.wait()
-        gzip_proc.wait()
-
         for filename in self._initrd_injections:
             logging.debug("Copying %s to the initrd." % filename)
             shutil.copy(filename, tempdir)
 
-        logging.debug("Repacking the initrd.")
+        logging.debug("Appending to the initrd.")
         find_proc = subprocess.Popen(['find', '.', '-print0'],
                                      stdout=subprocess.PIPE,
                                      stderr=sys.stderr, cwd=tempdir)
-        cpio_proc = subprocess.Popen(['cpio', '-o', '--null', '-c', '--quiet'],
+        cpio_proc = subprocess.Popen(['cpio', '-o', '--null', '-Hnewc', '--quiet'],
                                      stdin=find_proc.stdout,
                                      stdout=subprocess.PIPE,
                                      stderr=sys.stderr, cwd=tempdir)
-        new_initrd = initrd + '.new'
-        f = open(new_initrd, 'w')
+        f = open(initrd, 'ab')
         gzip_proc = subprocess.Popen(['gzip'], stdin=cpio_proc.stdout,
                                      stdout=f, stderr=sys.stderr)
-        f.close()
         cpio_proc.wait()
         find_proc.wait()
         gzip_proc.wait()
-        os.rename(new_initrd, initrd)
+        f.close()
         shutil.rmtree(tempdir)
 
     def _prepare_kernel_and_initrd(self, guest, meter):
@@ -233,7 +233,8 @@ class DistroInstaller(Installer.Installer):
                                readOnly=True,
                                transient=True)
 
-        if self._install_bootconfig.kernel:
+        # Make sure we always fetch kernel here if required
+        if self._install_bootconfig.kernel and not self.scratchdir_required():
             return disk
 
         # Need to fetch the kernel & initrd from a remote site, or
@@ -266,8 +267,11 @@ class DistroInstaller(Installer.Installer):
 
         return disk
 
+    def _persistent_cd(self):
+        return (self._location_is_path and self.cdrom and self.livecd)
+
     def _get_bootdev(self, isinstall, guest):
-        if isinstall:
+        if isinstall or self._persistent_cd():
             bootdev = self.bootconfig.BOOT_DEVICE_CDROM
         else:
             bootdev = self.bootconfig.BOOT_DEVICE_HARDDISK
@@ -276,6 +280,9 @@ class DistroInstaller(Installer.Installer):
     # General Installer methods
 
     def scratchdir_required(self):
+        if not self.location:
+            return False
+
         is_url = not self._location_is_path
         mount_dvd = self._location_is_path and not self.cdrom
 
@@ -297,6 +304,15 @@ class DistroInstaller(Installer.Installer):
         if dev:
             self.install_devices.append(dev)
 
+    def check_location(self):
+        if self._location_is_path:
+            # We already mostly validated this
+            return True
+        else:
+            # This will throw an error for us
+            OSDistro.detectMediaDistro(location=self.location, arch=self.arch)
+        return True
+
     def detect_distro(self):
         try:
             dist_info = OSDistro.detectMediaDistro(location=self.location,
