@@ -44,9 +44,11 @@ class ImageFetcher:
         else:
             path = self.location
 
-        if not path.endswith("/"):
-            path += "/"
-        path += filename
+        if filename:
+            if not path.endswith("/"):
+                path += "/"
+            path += filename
+
         return path
 
     def saveTemp(self, fileobj, prefix):
@@ -82,11 +84,11 @@ class ImageFetcher:
 
             try:
                 f = grabber.urlopen(path,
-                                    progress_obj = progresscb,
-                                    text = _("Retrieving file %s...") % base)
+                                    progress_obj=progresscb,
+                                    text=_("Retrieving file %s...") % base)
             except Exception, e:
-                raise ValueError, _("Couldn't acquire file %s: %s") % \
-                                    (path, str(e))
+                raise ValueError(_("Couldn't acquire file %s: %s") %
+                                   (path, str(e)))
 
             tmpname = self.saveTemp(f, prefix=base + ".")
             logging.debug("Saved file to " + tmpname)
@@ -204,4 +206,3 @@ class DirectImageFetcher(LocalImageFetcher):
 
     def prepareLocation(self):
         self.srcdir = self.location
-
