@@ -19,19 +19,24 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import os, errno
+import os
+import errno
 import struct
 import platform
 import logging
+import copy
 
 import _util
 import virtinst
+import XMLBuilderDomain
+from XMLBuilderDomain import _xml_property
 from virtinst import CapabilitiesParser
 from virtinst import _virtinst as _
 from VirtualDisk import VirtualDisk
+from Boot import Boot
 
-XEN_SCRATCH="/var/lib/xen"
-LIBVIRT_SCRATCH="/var/lib/libvirt/boot"
+XEN_SCRATCH = "/var/lib/xen"
+LIBVIRT_SCRATCH = "/var/lib/libvirt/boot"
 
 def _get_scratchdir(typ):
     scratch = None
@@ -52,7 +57,7 @@ def _get_scratchdir(typ):
 
     return scratch
 
-class Installer(object):
+class Installer(XMLBuilderDomain.XMLBuilderDomain):
     """
     Installer classes attempt to encapsulate all the parameters needed
     to 'install' a guest: essentially, booting the guest with the correct
@@ -77,30 +82,36 @@ class Installer(object):
         - Hypervisor name (parameter 'type') ('qemu', 'kvm', 'xen', etc.)
         - Guest architecture ('i686', 'x86_64')
     """
-    def __init__(self, type = "xen", location = None, boot = None,
-                 extraargs = None, os_type = None, conn = None):
+
+    _dumpxml_xpath = "/domain/os"
+    def __init__(self, type="xen", location=None, boot=None,
+                 extraargs=None, os_type=None, conn=None,
+                 parsexml=None, parsexmlnode=None, caps=None):
+        XMLBuilderDomain.XMLBuilderDomain.__init__(self, conn, parsexml,
+                                                   parsexmlnode, caps=caps)
+
         self._type = None
         self._location = None
-        self._extraargs = None
-        self._boot = None
+        self._initrd_injections = []
         self._cdrom = False
-        # XXX: We should set this default based on capabilities?
-        self._os_type = "xen"
-        self._conn = conn
+        self._os_type = None
         self._scratchdir = None
+        self._arch = None
+        self._machine = None
+        self._loader = None
+        self._install_bootconfig = Boot(self.conn)
+        self._bootconfig = Boot(self.conn, parsexml, parsexmlnode)
 
-        # VirtualDisk that contains install media
-        self._install_disk = None
+        # Devices created/added during the prepare() stage
+        self.install_devices = []
 
-        if self.conn:
-            self._caps = CapabilitiesParser.parse(self.conn.getCapabilities())
+        if self._is_parse():
+            return
 
-            # FIXME: Better solution? Skip validating this since we may not be
-            # able to install a VM of the host arch
-            self._arch = self._caps.host.arch
-        else:
-            self._caps = None
-            self._arch = None
+        # FIXME: Better solution? Skip validating this since we may not be
+        # able to install a VM of the host arch
+        if self._get_caps():
+            self._arch = self._get_caps().host.arch
 
         if type is None:
             type = "xen"
@@ -108,29 +119,32 @@ class Installer(object):
 
         if not os_type is None:
             self.os_type = os_type
+        else:
+            self.os_type = "xen"
         if not location is None:
             self.location = location
+
         if not boot is None:
             self.boot = boot
-        if not extraargs is None:
-            self.extraargs = extraargs
+        self.extraargs = extraargs
 
         self._tmpfiles = []
 
-    def get_install_disk(self):
-        return self._install_disk
-    install_disk = property(get_install_disk)
-
     def get_conn(self):
         return self._conn
     conn = property(get_conn)
 
+    def _get_bootconfig(self):
+        return self._bootconfig
+    bootconfig = property(_get_bootconfig)
+
     # Hypervisor name (qemu, kvm, xen, lxc, etc.)
     def get_type(self):
         return self._type
     def set_type(self, val):
         self._type = val
-    type = property(get_type, set_type)
+    type = _xml_property(get_type, set_type,
+                         xpath="./@type")
 
     # Virtualization type ('xen' == xen paravirt, or 'hvm)
     def get_os_type(self):
@@ -143,7 +157,8 @@ class Installer(object):
 
         # XXX: Need to validate this: have some whitelist based on caps?
         self._os_type = val
-    os_type = property(get_os_type, set_os_type)
+    os_type = _xml_property(get_os_type, set_os_type,
+                            xpath="./os/type")
 
     def get_arch(self):
         return self._arch
@@ -151,7 +166,22 @@ class Installer(object):
         # XXX: Sanitize to a consisten value (i368 -> i686)
         # XXX: Validate against caps
         self._arch = val
-    arch = property(get_arch, set_arch)
+    arch = _xml_property(get_arch, set_arch,
+                         xpath="./os/type/@arch")
+
+    def _get_machine(self):
+        return self._machine
+    def _set_machine(self, val):
+        self._machine = val
+    machine = _xml_property(_get_machine, _set_machine,
+                            xpath="./os/type/@machine")
+
+    def _get_loader(self):
+        return self._loader
+    def _set_loader(self, val):
+        self._loader = val
+    loader = _xml_property(_get_loader, _set_loader,
+                           xpath="./os/loader")
 
     def get_scratchdir(self):
         if not self.scratchdir_required():
@@ -166,7 +196,7 @@ class Installer(object):
         return self._cdrom
     def set_cdrom(self, enable):
         if enable not in [True, False]:
-            raise ValueError, _("Guest.cdrom must be a boolean type")
+            raise ValueError(_("Guest.cdrom must be a boolean type"))
         self._cdrom = enable
     cdrom = property(get_cdrom, set_cdrom)
 
@@ -176,33 +206,49 @@ class Installer(object):
         self._location = val
     location = property(get_location, set_location)
 
+    def get_initrd_injections(self):
+        return self._initrd_injections
+    def set_initrd_injections(self, val):
+        self._initrd_injections = val
+    initrd_injections = property(get_initrd_injections, set_initrd_injections)
+
     # kernel + initrd pair to use for installing as opposed to using a location
     def get_boot(self):
-        return self._boot
+        return {"kernel" : self._install_bootconfig.kernel,
+                "initrd" : self._install_bootconfig.initrd}
     def set_boot(self, val):
         self.cdrom = False
+        boot = {}
         if type(val) == tuple:
             if len(val) != 2:
-                raise ValueError, _("Must pass both a kernel and initrd")
+                raise ValueError(_("Must pass both a kernel and initrd"))
             (k, i) = val
-            self._boot = {"kernel": k, "initrd": i}
+            boot = {"kernel": k, "initrd": i}
+
         elif type(val) == dict:
-            if not val.has_key("kernel") or not val.has_key("initrd"):
-                raise ValueError, _("Must pass both a kernel and initrd")
-            self._boot = val
+            if "kernel" not in val or "initrd" not in val:
+                raise ValueError(_("Must pass both a kernel and initrd"))
+            boot = val
+
         elif type(val) == list:
             if len(val) != 2:
-                raise ValueError, _("Must pass both a kernel and initrd")
-            self._boot = {"kernel": val[0], "initrd": val[1]}
+                raise ValueError(_("Must pass both a kernel and initrd"))
+            boot = {"kernel": val[0], "initrd": val[1]}
+
         else:
-            raise ValueError, _("Kernel and initrd must be specified by a list, dict, or tuple.")
+            raise ValueError(_("Kernel and initrd must be specified by "
+                               "a list, dict, or tuple."))
+
+        self._install_bootconfig.kernel = boot.get("kernel")
+        self._install_bootconfig.initrd = boot.get("initrd")
+
     boot = property(get_boot, set_boot)
 
     # extra arguments to pass to the guest installer
     def get_extra_args(self):
-        return self._extraargs
+        return self._install_bootconfig.kernel_args
     def set_extra_args(self, val):
-        self._extraargs = val
+        self._install_bootconfig.kernel_args = val
     extraargs = property(get_extra_args, set_extra_args)
 
 
@@ -216,28 +262,33 @@ class Installer(object):
         return False
 
     # Private methods
+    def _get_bootdev(self, isinstall, guest):
+        raise NotImplementedError
 
-    def _get_osblob_helper(self, guest, isinstall, kernel=None, bootdev=None):
+    def _build_boot_order(self, isinstall, guest):
+        bootorder = [self._get_bootdev(isinstall, guest)]
 
-        # TODO: kernel should go away: we should be able to pull this
-        #       directly from the installer. This may mean deprecating
-        #       extraargs or something
+        # If guest has an attached disk, always have 'hd' in the boot
+        # list, so disks are marked as bootable/installable (needed for
+        # windows virtio installs, and booting local disk from PXE)
+        for disk in guest.get_devices("disk"):
+            if disk.device == disk.DEVICE_DISK:
+                bootdev = self.bootconfig.BOOT_DEVICE_HARDDISK
+                if bootdev not in bootorder:
+                    bootorder.append(bootdev)
+                break
 
-        def get_param(obj, paramname):
-            if hasattr(obj, paramname):
-                return getattr(obj, paramname)
-            return None
-
-        ishvm = False
-        if isinstance(guest, virtinst.FullVirtGuest):
-            ishvm = True
+        return bootorder
 
+    def _get_osblob_helper(self, guest, isinstall, bootconfig):
+        ishvm = self.os_type == "hvm"
         conn = guest.conn
         arch = self.arch
-        loader = get_param(guest, "loader")
+        loader = self.loader
+        if not loader and ishvm and self.type == "xen":
+            loader = "/usr/lib/xen/boot/hvmloader"
 
-        osblob = ""
-        if not isinstall and not ishvm:
+        if not isinstall and not ishvm and not self.bootconfig.kernel:
             return "<bootloader>%s</bootloader>" % _util.pygrub_path(conn)
 
         osblob = "<os>\n"
@@ -248,29 +299,25 @@ class Installer(object):
         if os_type == "xen" and self.type == "xen":
             os_type = "linux"
 
+        osblob += "    <type"
         if arch:
-            osblob += "    <type arch='%s'>%s</type>\n" % (arch, os_type)
-        else:
-            osblob += "    <type>%s</type>\n" % os_type
+            osblob += " arch='%s'" % arch
+        if self.machine:
+            osblob += " machine='%s'" % self.machine
+        osblob += ">%s</type>\n" % os_type
 
         if loader:
             osblob += "    <loader>%s</loader>\n" % loader
 
-        if isinstall and kernel and kernel["kernel"]:
-            osblob += "    <kernel>%s</kernel>\n"   % _util.xml_escape(kernel["kernel"])
-            osblob += "    <initrd>%s</initrd>\n"   % _util.xml_escape(kernel["initrd"])
-            osblob += "    <cmdline>%s</cmdline>\n" % _util.xml_escape(kernel["extraargs"])
-        elif bootdev is not None:
-            osblob += "    <boot dev='%s'/>\n" % bootdev
-
-        osblob += "  </os>"
+        osblob += bootconfig.get_xml_config()
+        osblob = _util.xml_append(osblob, "  </os>")
 
         return osblob
 
 
     # Method definitions
 
-    def get_install_xml(self, guest, isinstall):
+    def _get_xml_config(self, guest, isinstall):
         """
         Generate the portion of the guest xml that determines boot devices
         and parameters. (typically the <os></os> block)
@@ -281,8 +328,28 @@ class Installer(object):
                           'post-install' phase.
         @type isinstall: C{bool}
         """
-        # Must be implemented in sub class
-        raise NotImplementedError
+        if isinstall:
+            bootconfig = self._install_bootconfig
+        else:
+            bootconfig = self.bootconfig
+
+        if isinstall and not self.has_install_phase():
+            return
+
+        bootorder = self._build_boot_order(isinstall, guest)
+        bootconfig = copy.copy(bootconfig)
+        if not bootconfig.bootorder:
+            bootconfig.bootorder = bootorder
+
+        return self._get_osblob_helper(guest, isinstall, bootconfig)
+
+    def has_install_phase(self):
+        """
+        Return True if the requested setup is actually installing an OS
+        into the guest. Things like LiveCDs, Import, or a manually specified
+        bootorder do not have an install phase.
+        """
+        return True
 
     def cleanup(self):
         """
@@ -292,16 +359,15 @@ class Installer(object):
             logging.debug("Removing " + f)
             os.unlink(f)
         self._tmpfiles = []
+        self.install_devices = []
 
-    def prepare(self, guest, meter, distro=None):
+    def prepare(self, guest, meter):
         """
         Fetch any files needed for installation.
         @param guest: guest instance being installed
         @type L{Guest}
         @param meter: progress meter
         @type Urlgrabber ProgressMeter
-        @param distro: Name of distro being installed
-        @type C{str} name from Guest os dictionary
         """
         raise NotImplementedError("Must be implemented in subclass")
 
@@ -316,22 +382,31 @@ class Installer(object):
             # XXX: Use block peek for this?
             return True
 
-        if len(guest.disks) == 0 \
-           or guest.disks[0].device != VirtualDisk.DEVICE_DISK:
+        if (len(guest.disks) == 0 or
+            guest.disks[0].device != VirtualDisk.DEVICE_DISK):
             return True
 
-        if _util.is_vdisk(guest.disks[0].path):
+        disk = guest.disks[0]
+
+        if _util.is_vdisk(disk.path):
+            return True
+
+        if (disk.driver_type and
+            disk.driver_type not in [disk.DRIVER_TAP_RAW,
+                                     disk.DRIVER_QEMU_RAW]):
+            # Might be a non-raw format
             return True
 
         # Check for the 0xaa55 signature at the end of the MBR
         try:
-            fd = os.open(guest.disks[0].path, os.O_RDONLY)
+            fd = os.open(disk.path, os.O_RDONLY)
         except OSError, (err, msg):
             logging.debug("Failed to open guest disk: %s" % msg)
             if err == errno.EACCES and os.geteuid() != 0:
                 return True # non root might not have access to block devices
             else:
                 raise
+
         buf = os.read(fd, 512)
         os.close(fd)
         return (len(buf) == 512 and
@@ -352,32 +427,32 @@ class Installer(object):
         Return a L{Guest} instance wrapping the current installer.
 
         If all the appropriate values are present in the installer
-        (conn, type, os_type, arch), we have everything we need to determine
-        what L{Guest} class is expected and what default values to pass
-        it. This is a convenience method to save the API user from having
-        to enter all these known details twice.
+        (conn, type, os_type, arch, machine), we have everything we need
+        to determine what L{Guest} class is expected and what default values
+        to pass it. This is a convenience method to save the API user from
+        having to enter all these known details twice.
         """
 
         if not self.conn:
             raise ValueError(_("A connection must be specified."))
 
         guest, domain = CapabilitiesParser.guest_lookup(conn=self.conn,
-                                                        caps=self._caps,
+                                                        caps=self._get_caps(),
                                                         os_type=self.os_type,
                                                         type=self.type,
-                                                        arch=self.arch)
+                                                        arch=self.arch,
+                                                        machine=self.machine)
 
-        if self.os_type == "xen":
-            gobj = virtinst.ParaVirtGuest(installer=self, connection=self.conn)
-            gobj.arch = guest.arch
-        elif self.os_type == "hvm":
-            gobj = virtinst.FullVirtGuest(installer=self,
-                                          connection=self.conn,
-                                          emulator=domain.emulator,
-                                          arch=guest.arch)
-            gobj.loader = domain.loader
-        else:
+        if self.os_type not in ["xen", "hvm"]:
             raise ValueError(_("No 'Guest' class for virtualization type '%s'"
                              % self.type))
 
+        gobj = virtinst.Guest(installer=self, connection=self.conn)
+        gobj.arch = guest.arch
+        gobj.emulator = domain.emulator
+        self.loader = domain.loader
+
         return gobj
+
+# Back compat
+Installer.get_install_xml = Installer.get_xml_config
