@@ -28,7 +28,7 @@ import socket
 import ConfigParser
 
 import virtinst
-from virtinst.Guest import Guest
+import osdict
 from virtinst import _util
 from virtinst import _virtinst as _
 
@@ -106,60 +106,81 @@ def _storeForDistro(fetcher, baseuri, typ, progresscb, arch, distro=None,
         if store.isValidStore(fetcher, progresscb):
             return store
 
-    raise ValueError, _("Could not find an installable distribution at '%s'" %
-                        baseuri)
+    raise ValueError(
+        _("Could not find an installable distribution at '%s'\n"
+          "The location must be the root directory of an install tree." %
+          baseuri))
 
-def _acquireMedia(iskernel, guest, baseuri, progresscb, arch,
-                  scratchdir="/var/tmp", _type=None, distro=None):
+def _locationCheckWrapper(guest, baseuri, progresscb,
+                          scratchdir, _type, arch, callback):
     fetcher = _fetcherForURI(baseuri, scratchdir)
+    if guest:
+        arch = guest.arch
 
     try:
         fetcher.prepareLocation()
     except ValueError, e:
-        raise ValueError, _("Invalid install location: ") + str(e)
+        raise ValueError(_("Invalid install location: ") + str(e))
 
     try:
         store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, typ=_type,
-                                progresscb=progresscb, distro=distro,
-                                scratchdir=scratchdir, arch=arch)
+                                progresscb=progresscb, scratchdir=scratchdir,
+                                arch=arch)
 
-        if iskernel is True:
-            # FIXME: We should probably do this for both kernel and boot
-            # disk?
-            os_type, os_variant = store.get_osdict_info()
-            return (store.acquireKernel(guest, fetcher, progresscb),
-                    os_type, os_variant)
-        elif iskernel is False:
-            return store.acquireBootDisk(fetcher, progresscb)
-        else:
-            # Kind of a hack. Just return the store for detectDistro
-            return store
+        return callback(store, fetcher)
     finally:
         fetcher.cleanupLocation()
 
+def _acquireMedia(iskernel, guest, baseuri, progresscb,
+                  scratchdir="/var/tmp", _type=None):
+
+    def media_cb(store, fetcher):
+        os_type, os_variant = store.get_osdict_info()
+        media = None
+
+        if iskernel:
+            media = store.acquireKernel(guest, fetcher, progresscb)
+        else:
+            media = store.acquireBootDisk(guest, fetcher, progresscb)
+
+        return [store, os_type, os_variant, media]
+
+    return _locationCheckWrapper(guest, baseuri, progresscb, scratchdir, _type,
+                                 None, media_cb)
+
 # Helper method to lookup install media distro and fetch an install kernel
-def acquireKernel(guest, baseuri, progresscb, arch,
-                  scratchdir="/var/tmp", type=None, distro=None):
-    return _acquireMedia(True, guest, baseuri, progresscb, arch,
-                         scratchdir, type, distro)
+def acquireKernel(guest, baseuri, progresscb, scratchdir, type=None):
+    iskernel = True
+    return _acquireMedia(iskernel, guest, baseuri, progresscb,
+                         scratchdir, type)
 
 # Helper method to lookup install media distro and fetch a boot iso
-def acquireBootDisk(baseuri, progresscb, arch, scratchdir="/var/tmp",
-                    type=None, distro=None):
-    return _acquireMedia(False, None, baseuri, progresscb, arch,
-                         scratchdir, type, distro)
+def acquireBootDisk(guest, baseuri, progresscb, scratchdir, type=None):
+    iskernel = False
+    return _acquireMedia(iskernel, guest, baseuri, progresscb,
+                         scratchdir, type)
 
 def _check_ostype_valid(os_type):
-    return bool(os_type in Guest.list_os_types())
+    return bool(os_type in osdict.sort_helper(osdict.OS_TYPES))
+
 def _check_osvariant_valid(os_type, os_variant):
     return bool(_check_ostype_valid(os_type) and
-                os_variant in Guest.list_os_variants(os_type))
+        os_variant in osdict.sort_helper(osdict.OS_TYPES[os_type]["variants"]))
 
 # Attempt to detect the os type + variant for the passed location
 def detectMediaDistro(location, arch):
     import urlgrabber
-    progress = urlgrabber.progress.BaseMeter()
-    store = _acquireMedia(None, None, location, progress, arch, "/var/tmp")
+    progresscb = urlgrabber.progress.BaseMeter()
+    guest = None
+    baseuri = location
+    scratchdir = "/var/tmp"
+    _type = None
+    def media_cb(store, ignore):
+        return store
+
+    store = _locationCheckWrapper(guest, baseuri, progresscb, scratchdir,
+                                  _type, arch, media_cb)
+
     return store.get_osdict_info()
 
 
@@ -254,7 +275,7 @@ class Distro:
         return self._kernelFetchHelper(fetcher, guest, progresscb, kernelpath,
                                        initrdpath)
 
-    def acquireBootDisk(self, fetcher, progresscb):
+    def acquireBootDisk(self, guest, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             return fetcher.acquireFile(self._getTreeinfoMedia("boot.iso"),
                                        progresscb)
@@ -432,7 +453,7 @@ class GenericDistro(Distro):
                                        self._valid_kernel_path[0],
                                        self._valid_kernel_path[1])
 
-    def acquireBootDisk(self, fetcher, progresscb):
+    def acquireBootDisk(self, guest, fetcher, progresscb):
         if self._valid_iso_path == None:
             raise ValueError(_("Could not find a boot iso path for this tree."))
 
@@ -482,7 +503,6 @@ class FedoraDistro(RedHatDistro):
             return False
 
     def _latestFedoraVariant(self):
-        import osdict
         ret = None
         for var in osdict.sort_helper(osdict.OS_TYPES["linux"]["variants"]):
             if var.startswith("fedora"):
@@ -515,8 +535,12 @@ class RHELDistro(RedHatDistro):
                 self.os_variant = "rhel5"
                 return True
             if fetcher.hasFile("RedHat"):
-                logging.debug("Detected a RHEL 4 distro")
-                self.os_variant = "rhel4"
+                if fetcher.hasFile("dosutils"):
+                    self.os_variant = "rhel3"
+                else:
+                    self.os_variant = "rhel4"
+
+                logging.debug("Detected a %s distro" % self.os_variant)
                 return True
             return False
 
@@ -699,7 +723,7 @@ class SuseDistro(Distro):
     # install-initrd RPM entries - capturing the directory they are
     # in and the version'd filename.
     def _extractRPMNames(self, filelist):
-        filelistData = gzip.GzipFile(filelist, mode = "r")
+        filelistData = gzip.GzipFile(filelist, mode="r")
         try:
             arches = [self.arch]
             # On i686 arch, we also look under i585 and i386 dirs
@@ -796,7 +820,7 @@ class SuseDistro(Distro):
                     line = fn.readline()
                     if not line:
                         break
-                    line = line[:len(line)-1]
+                    line = line[:len(line) - 1]
                     modnames.append(line)
             finally:
                 fn.close()
@@ -822,7 +846,7 @@ class SuseDistro(Distro):
 
             # Copy modules we need into initrd staging dir
             for modname in modnames:
-                if modpaths.has_key(modname):
+                if modname in modpaths:
                     src = modpaths[modname]
                     dst = moddir + "/" + modname
                     os.system("cp " + src + " " + dst)
@@ -982,7 +1006,7 @@ class SunDistro(Distro):
         """Determine if uri points to a tree of the store's distro"""
         raise NotImplementedError
 
-    def acquireBootDisk(self, fetcher, progresscb):
+    def acquireBootDisk(self, guest, fetcher, progresscb):
         return fetcher.acquireFile("images/solarisdvd.iso", progresscb)
 
     def process_extra_args(self, argstr):
@@ -1008,7 +1032,7 @@ class SunDistro(Distro):
                     Bargs = args[i]
                 else:
                     Bargs = ','.join([Bargs, args[i]])
-        
+
             elif exarg == '-m':
                 i += 1
                 if i == len(args):
@@ -1121,7 +1145,7 @@ class SolarisDistro(SunDistro):
 class OpenSolarisDistro(SunDistro):
 
     os_variant = "opensolaris"
- 
+
     kernelpath = "platform/i86xpv/kernel/unix"
     initrdpaths = [ "platform/i86pc/boot_archive", "boot/x86.microroot" ]
 
