@@ -19,6 +19,7 @@
 
 import VirtualDevice
 from virtinst import _virtinst as _
+from XMLBuilderDomain import _xml_property
 
 class VirtualAudio(VirtualDevice.VirtualDevice):
 
@@ -27,10 +28,14 @@ class VirtualAudio(VirtualDevice.VirtualDevice):
     MODEL_DEFAULT = "default"
     MODELS = [ "es1370", "sb16", "pcspk", "ac97", MODEL_DEFAULT ]
 
-    def __init__(self, model=None, conn=None):
-        VirtualDevice.VirtualDevice.__init__(self, conn)
+    def __init__(self, model=None, conn=None,
+                 parsexml=None, parsexmlnode=None, caps=None):
+        VirtualDevice.VirtualDevice.__init__(self, conn,
+                                             parsexml, parsexmlnode, caps)
 
         self._model = None
+        if self._is_parse():
+            return
 
         if model == None:
             model = self.MODEL_DEFAULT
@@ -40,14 +45,15 @@ class VirtualAudio(VirtualDevice.VirtualDevice):
         return self._model
     def set_model(self, new_model):
         if type(new_model) != str:
-            raise ValueError, _("'model' must be a string, "
-                                " was '%s'." % type(new_model))
+            raise ValueError(_("'model' must be a string, "
+                               " was '%s'." % type(new_model)))
         if not self.MODELS.count(new_model):
-            raise ValueError, _("Unsupported sound model '%s'" % new_model)
+            raise ValueError(_("Unsupported sound model '%s'" % new_model))
         self._model = new_model
-    model = property(get_model, set_model)
+    model = _xml_property(get_model, set_model,
+                          xpath="./@model")
 
-    def get_xml_config(self):
+    def _get_xml_config(self):
         model = self.model
         if model == self.MODEL_DEFAULT:
             model = "es1370"
