@@ -19,13 +19,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import libvirt
-
-import CapabilitiesParser
-import _util
+import XMLBuilderDomain
 from virtinst import _virtinst as _
 
-class VirtualDevice(object):
+class VirtualDevice(XMLBuilderDomain.XMLBuilderDomain):
     """
     Base class for all domain xml device objects.
     """
@@ -58,18 +55,22 @@ class VirtualDevice(object):
                             VIRTUAL_DEV_AUDIO,
                             VIRTUAL_DEV_VIDEO,
                             VIRTUAL_DEV_HOSTDEV,
-                            VIRTUAL_DEV_WATCHDOG ]
+                            VIRTUAL_DEV_WATCHDOG]
 
     # General device type (disk, interface, etc.)
     _virtual_device_type = None
 
-    def __init__(self, conn=None):
+    def __init__(self, conn=None, parsexml=None, parsexmlnode=None, caps=None):
         """
         Initialize device state
 
         @param conn: libvirt connection to validate device against
         @type conn: virConnect
         """
+        XMLBuilderDomain.XMLBuilderDomain.__init__(self, conn, parsexml,
+                                                   parsexmlnode,
+                                                   caps=None)
+
         if not self._virtual_device_type:
             raise ValueError(_("Virtual device type must be set in subclass."))
 
@@ -77,60 +78,19 @@ class VirtualDevice(object):
             raise ValueError(_("Unknown virtual device type '%s'.") %
                              self._virtual_device_type)
 
-        if conn:
-            if not isinstance(conn, libvirt.virConnect):
-                raise ValueError, _("'conn' must be a virConnect instance")
-        self._conn = conn
-
-        self.__remote = None
-        if self.conn:
-            self.__remote = _util.is_uri_remote(self.conn.getURI())
-
-        self._caps = None
-        if self.conn:
-            self._caps = CapabilitiesParser.parse(self.conn.getCapabilities())
-
-    def get_conn(self):
-        return self._conn
-    def set_conn(self, val):
-        if not isinstance(val, libvirt.virConnect):
-            raise ValueError(_("'conn' must be a virConnect instance."))
-        self._conn = val
-    conn = property(get_conn, set_conn)
 
     def get_virtual_device_type(self):
         return self._virtual_device_type
     virtual_device_type = property(get_virtual_device_type)
 
-    def _is_remote(self):
-        return bool(self.__remote)
-
-    def _get_uri(self):
-        if self.conn:
-            return self.conn.getURI()
-        return None
-
-    def _check_bool(self, val, name):
-        if val not in [True, False]:
-            raise ValueError, _("'%s' must be True or False" % name)
-
-    def _check_str(self, val, name):
-        if type(val) is not str:
-            raise ValueError, _("'%s' must be a string, not '%s'." %
-                                (name, type(val)))
-
-    def get_xml_config(self):
-        """
-        Construct and return device xml
-
-        @return: device xml representation as a string
-        @rtype: str
-        """
+    def _get_xml_config(self):
+        # See XMLBuilderDomain for docs
         raise NotImplementedError()
 
     def setup_dev(self, conn=None, meter=None):
         """
-        Perform any necessary device initialization
+        Perform potentially hazardous device initialization, like
+        storage creation or host device reset
 
         @param conn: Optional connection to use if neccessary. If not
                      specified, device's 'conn' will be used
