@@ -19,30 +19,101 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import os, sys
+import os
+import sys
 import logging
 import logging.handlers
-import gettext, locale
+import gettext
+import locale
 import optparse
-from optparse import OptionValueError, OptionParser
+from optparse import OptionValueError, OptionParser, OptionGroup
+import re
+import difflib
+import tempfile
 
 import libvirt
 import _util
 import virtinst
-from virtinst import VirtualNetworkInterface, VirtualVideoDevice, Guest, \
+from _util import listify
+from virtinst import VirtualNetworkInterface, Guest, \
                      VirtualGraphics, VirtualAudio, VirtualDisk, User
 from virtinst import _virtinst as _
 
 MIN_RAM = 64
 force = False
+quiet = False
 doprompt = True
 
-def check_if_test_uri_remote(uri):
-    magic = "__virtinst_test_remote__"
-    if uri and uri.startswith(magic):
-        uri = uri.replace(magic, "")
+log_exception = _util.log_exception
+_virtinst_uri_magic = "__virtinst_test__"
+
+def _is_virtinst_test_uri(uri):
+    return uri and uri.startswith(_virtinst_uri_magic)
+
+def _open_test_uri(uri):
+    """
+    This hack allows us to fake various drivers via passing a magic
+    URI string to virt-*. Helps with testing
+    """
+    uri = uri.replace(_virtinst_uri_magic, "")
+    ret = uri.split(",", 1)
+    uri = ret[0]
+    ignore, opts = parse_optstr(len(ret) > 1 and ret[1] or "")
+
+    conn = open_connection(uri)
+
+    def sanitize_xml(xml):
+        orig = xml
+        xml = re.sub("arch='.*'", "arch='i686'", xml)
+        xml = re.sub("domain type='.*'", "domain type='test'", xml)
+        xml = re.sub("machine type='.*'", "", xml)
+
+        logging.debug("virtinst test sanitizing diff\n:%s" %
+                      "\n".join(difflib.unified_diff(orig.split("\n"),
+                                                     xml.split("\n"))))
+        return xml
+
+    # Need tmpfile names to be deterministic
+    if "predictable" in opts:
+        def fakemkstemp(prefix, *args, **kwargs):
+            ignore = args
+            ignore = kwargs
+            filename = os.path.join(".", prefix)
+            return os.open(filename, os.O_RDWR | os.O_CREAT), filename
+        tempfile.mkstemp = fakemkstemp
+
+        _util.randomMAC = lambda type_: "00:11:22:33:44:55"
+        _util.uuidToString = lambda r: "00000000-1111-2222-3333-444444444444"
+
+    # Fake remote status
+    if "remote" in opts:
         _util.is_uri_remote = lambda uri_: True
-    return uri
+
+    # Fake capabilities
+    if "caps" in opts:
+        capsxml = file(opts["caps"]).read()
+        conn.getCapabilities = lambda: capsxml
+
+    if "qemu" in opts or "xen" in opts:
+        conn.getVersion = lambda: 10000000000
+
+        origcreate = conn.createLinux
+        origdefine = conn.defineXML
+        def newcreate(xml, flags):
+            xml = sanitize_xml(xml)
+            origcreate(xml, flags)
+        def newdefine(xml):
+            xml = sanitize_xml(xml)
+            origdefine(xml)
+        conn.createLinux = newcreate
+        conn.defineXML = newdefine
+
+        if "qemu" in opts:
+            conn.getURI = lambda: "qemu+abc:///system"
+        if "xen" in opts:
+            conn.getURI = lambda: "xen+abc:///"
+
+    return conn
 
 class VirtStreamHandler(logging.StreamHandler):
 
@@ -74,6 +145,8 @@ class VirtOptionParser(OptionParser):
         encoding = getattr(f, "encoding", None)
         if not encoding:
             (dummy, encoding) = locale.getlocale()
+        if not encoding:
+            encoding = "UTF-8"
         return encoding
 
     def print_help(self, file=None):
@@ -133,8 +206,13 @@ def setupGettext():
     gettext.bindtextdomain(virtinst.gettext_app, virtinst.gettext_dir)
     gettext.install(virtinst.gettext_app, virtinst.gettext_dir)
 
-def setupLogging(appname, debug=False):
-    # set up logging
+def earlyLogging():
+    logging.basicConfig(level=logging.DEBUG, format='%(message)s')
+
+def setupLogging(appname, debug=False, do_quiet=False):
+    global quiet
+    quiet = do_quiet
+
     vi_dir = os.path.expanduser("~/.virtinst")
     if not os.access(vi_dir, os.W_OK):
         if os.path.exists(vi_dir):
@@ -148,15 +226,21 @@ def setupLogging(appname, debug=False):
 
 
     dateFormat = "%a, %d %b %Y %H:%M:%S"
-    fileFormat = "[%(asctime)s " + appname + " %(process)d] %(levelname)s (%(module)s:%(lineno)d) %(message)s"
+    fileFormat = ("[%(asctime)s " + appname + " %(process)d] "
+                  "%(levelname)s (%(module)s:%(lineno)d) %(message)s")
     streamDebugFormat = "%(asctime)s %(levelname)-8s %(message)s"
     streamErrorFormat = "%(levelname)-8s %(message)s"
     filename = os.path.join(vi_dir, appname + ".log")
 
     rootLogger = logging.getLogger()
+
+    # Undo early logging
+    for handler in rootLogger.handlers:
+        rootLogger.removeHandler(handler)
+
     rootLogger.setLevel(logging.DEBUG)
-    fileHandler = logging.handlers.RotatingFileHandler(filename, "a",
-                                                       1024*1024, 5)
+    fileHandler = logging.handlers.RotatingFileHandler(filename, "ae",
+                                                       1024 * 1024, 5)
 
     fileHandler.setFormatter(logging.Formatter(fileFormat,
                                                dateFormat))
@@ -168,7 +252,11 @@ def setupLogging(appname, debug=False):
         streamHandler.setFormatter(logging.Formatter(streamDebugFormat,
                                                      dateFormat))
     else:
-        streamHandler.setLevel(logging.ERROR)
+        if quiet:
+            level = logging.ERROR
+        else:
+            level = logging.WARN
+        streamHandler.setLevel(level)
         streamHandler.setFormatter(logging.Formatter(streamErrorFormat))
     rootLogger.addHandler(streamHandler)
 
@@ -193,28 +281,49 @@ def setupLogging(appname, debug=False):
 def fail(msg, do_exit=True):
     """Convenience function when failing in cli app"""
     logging.error(msg)
-    _util.log_exception()
+    log_exception()
     if do_exit:
         _fail_exit()
 
+def print_stdout(msg, do_force=False):
+    if do_force or not quiet:
+        print msg
+
+def print_stderr(msg):
+    logging.debug(msg)
+    print >> sys.stderr, msg
+
 def _fail_exit():
     sys.exit(1)
 
 def nice_exit():
-    print _("Exiting at user request.")
+    print_stdout(_("Exiting at user request."))
     sys.exit(0)
 
+def virsh_start_cmd(guest):
+    return ("virsh --connect %s start %s" % (guest.conn.getURI(), guest.name))
+
+def install_fail(guest):
+    virshcmd = virsh_start_cmd(guest)
+
+    print_stderr(
+        _("Domain installation does not appear to have been successful.\n"
+          "If it was, you can restart your domain by running:\n"
+          "  %s\n"
+          "otherwise, please restart your installation.") % virshcmd)
+    sys.exit(1)
+
 # Connection opening helper functions
-def getConnection(connect):
-    if (connect and
-        not User.current().has_priv(User.PRIV_CREATE_DOMAIN, connect)):
+def getConnection(uri):
+    if (uri and not User.current().has_priv(User.PRIV_CREATE_DOMAIN, uri)):
         fail(_("Must be root to create Xen guests"))
 
-    # Hack to facilitate remote unit testing
-    connect = check_if_test_uri_remote(connect)
+    # Hack to facilitate virtinst unit testing
+    if _is_virtinst_test_uri(uri):
+        return _open_test_uri(uri)
 
-    logging.debug("Requesting libvirt URI %s" % (connect or "default"))
-    conn = open_connection(connect)
+    logging.debug("Requesting libvirt URI %s" % (uri or "default"))
+    conn = open_connection(uri)
     logging.debug("Received libvirt URI %s" % conn.getURI())
 
     return conn
@@ -235,7 +344,7 @@ def do_creds(creds, cbdata):
     try:
         return _do_creds(creds, cbdata)
     except:
-        _util.log_exception("Error in creds callback.")
+        log_exception("Error in creds callback.")
         raise
 
 def _do_creds(creds, cbdata_ignore):
@@ -334,7 +443,7 @@ def set_prompt(prompt=True):
 def is_prompt():
     return doprompt
 
-def prompt_for_input(noprompt_err, prompt = "", val = None, failed=False):
+def prompt_for_input(noprompt_err, prompt="", val=None, failed=False):
     if val is not None:
         return val
 
@@ -343,13 +452,9 @@ def prompt_for_input(noprompt_err, prompt = "", val = None, failed=False):
             # We already failed validation in a previous function, just exit
             _fail_exit()
 
-        msg = noprompt_err
-        if not force and not msg.count("--prompt"):
-            # msg wasn't already appended to from yes_or_no
-            msg = noprompt_err + " " + _("(use --prompt to run interactively)")
-        fail(msg)
+        fail(noprompt_err)
 
-    print prompt + " ",
+    print_stdout(prompt + " ", do_force=True)
     return sys.stdin.readline().strip()
 
 def yes_or_no_convert(s):
@@ -363,7 +468,7 @@ def yes_or_no_convert(s):
 def yes_or_no(s):
     ret = yes_or_no_convert(s)
     if ret == None:
-        raise ValueError, "A yes or no response is required"
+        raise ValueError(_("A yes or no response is required"))
     return ret
 
 def prompt_for_yes_or_no(warning, question):
@@ -407,24 +512,45 @@ def prompt_loop(prompt_txt, noprompt_err, passed_val, obj, param_name,
             passed_val = None
             failed = True
 
+def vcpu_cli_options(grp):
+    grp.add_option("", "--vcpus", type="string", dest="vcpus",
+        help=_("Number of vcpus to configure for your guest. Ex:\n"
+               "--vcpus 5\n"
+               "--vcpus 5,maxcpus=10\n"
+               "--vcpus sockets=2,cores=4,threads=2"))
+    grp.add_option("", "--cpuset", type="string", dest="cpuset",
+                   action="callback", callback=check_before_store,
+                   help=_("Set which physical CPUs Domain can use."))
+    grp.add_option("", "--cpu", type="string", dest="cpu",
+        action="callback", callback=check_before_store,
+        help=_("CPU model and features. Ex: --cpu coreduo,+x2apic"))
+    grp.add_option("", "--check-cpu", action="store_true", dest="check_cpu",
+                   help=optparse.SUPPRESS_HELP)
+
 # Register vnc + sdl options for virt-install and virt-image
 def graphics_option_group(parser):
-    from optparse import OptionGroup
 
     vncg = OptionGroup(parser, _("Graphics Configuration"))
+    vncg.add_option("", "--graphics", type="string", dest="graphics",
+                    action="callback", callback=check_before_store,
+      help=_("Specify display configuration. Ex:\n"
+             "--graphics vnc\n"
+             "--graphics spice,port=1,tlsport=2\n"
+             "--graphics none\n"
+             "--graphics vnc,password=foobar,port=5910,keymap=ja"))
     vncg.add_option("", "--vnc", action="store_true", dest="vnc",
-                    help=_("Use VNC for graphics support"))
+                    help=optparse.SUPPRESS_HELP)
     vncg.add_option("", "--vncport", type="int", dest="vncport",
-                    help=_("Port to use for VNC"))
+                    help=optparse.SUPPRESS_HELP)
     vncg.add_option("", "--vnclisten", type="string", dest="vnclisten",
-                    help=_("Address to listen on for VNC connections."))
+                    help=optparse.SUPPRESS_HELP)
     vncg.add_option("-k", "--keymap", type="string", dest="keymap",
                     action="callback", callback=check_before_store,
-                    help=_("set up keymap for the VNC console"))
+                    help=optparse.SUPPRESS_HELP)
     vncg.add_option("", "--sdl", action="store_true", dest="sdl",
-                    help=_("Use SDL for graphics support"))
+                    help=optparse.SUPPRESS_HELP)
     vncg.add_option("", "--nographics", action="store_true",
-                    help=_("Don't set up a graphical console for the guest."))
+                    help=optparse.SUPPRESS_HELP)
     return vncg
 
 # Specific function for disk prompting. Returns a validated VirtualDisk
@@ -457,7 +583,7 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
             msg = _("What would you like to use as the disk (file path)?")
             if not size is None:
                 msg = _("Please enter the path to the file you would like to "
-                        "use for storage. It will have size %sGB.") %(size,)
+                        "use for storage. It will have size %sGB.") % size
 
         if not no_path_needed:
             path = prompt_for_input(patherr, prompt_txt or msg, passed_path)
@@ -494,8 +620,10 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
         askmsg = _("Do you really want to use this disk (yes or no)")
 
         # Prompt if disk file already exists and preserve mode is not used
-        if ((path_exists and dev.type == dev.TYPE_FILE and warn_overwrite) or
-            is_prompt()):
+        does_collide = (path_exists and
+                        dev.type == dev.TYPE_FILE and
+                        dev.device == dev.DEVICE_DISK)
+        if (does_collide and (warn_overwrite or is_prompt())):
             msg = (_("This will overwrite the existing path '%s'" %
                    dev.path))
             if not prompt_for_yes_or_no(msg, askmsg):
@@ -521,17 +649,12 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
 # Ask for attributes
 #
 
-def listify(l):
-    if l is None:
-        return []
-    elif type(l) != list:
-        return [ l ]
-    else:
-        return l
+name_missing    = _("--name is required")
+ram_missing     = _("--ram amount in MB is required")
 
 def get_name(name, guest, image_name=None):
     prompt_txt = _("What is the name of your virtual machine?")
-    err_txt = _("A name is required for the virtual machine.")
+    err_txt = name_missing
 
     if name is None:
         name = image_name
@@ -539,7 +662,8 @@ def get_name(name, guest, image_name=None):
 
 def get_memory(memory, guest, image_memory=None):
     prompt_txt = _("How much RAM should be allocated (in megabytes)?")
-    err_txt = _("Memory amount is required for the virtual machine.")
+    err_txt = ram_missing
+
     def check_memory(mem):
         mem = int(mem)
         if mem < MIN_RAM:
@@ -548,7 +672,7 @@ def get_memory(memory, guest, image_memory=None):
         guest.memory = mem
 
     if memory is None and image_memory is not None:
-        memory = int(image_memory)/1024
+        memory = int(image_memory) / 1024
     prompt_loop(prompt_txt, err_txt, memory, guest, "memory",
                 func=check_memory)
 
@@ -559,7 +683,47 @@ def get_uuid(uuid, guest):
         except ValueError, e:
             fail(e)
 
-def get_vcpus(vcpus, check_cpu, guest, conn, image_vcpus=None):
+def _build_set_param(inst, opts):
+    def _set_param(paramname, keyname, val=None):
+        val = get_opt_param(opts, keyname, val)
+        if val == None:
+            return
+        setattr(inst, paramname, val)
+
+    return _set_param
+
+def parse_vcpu_option(guest, optstring, default_vcpus):
+    """
+    Helper to parse --vcpu string
+    """
+    vcpus, opts = parse_optstr(optstring, remove_first=True)
+    vcpus = vcpus or default_vcpus
+
+    set_param = _build_set_param(guest, opts)
+    set_cpu_param = _build_set_param(guest.cpu, opts)
+    has_vcpus = ("vcpus" in opts or vcpus)
+
+    set_param("vcpus", "vcpus", vcpus)
+    set_param("maxvcpus", "maxvcpus")
+
+    set_cpu_param("sockets", "sockets")
+    set_cpu_param("cores", "cores")
+    set_cpu_param("threads", "threads")
+
+    if not has_vcpus:
+        guest.vcpus = guest.cpu.vcpus_from_topology()
+
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
+
+def get_vcpus(vcpus, check_cpu, guest, image_vcpus=None):
+    if vcpus is None and image_vcpus is not None:
+        vcpus = int(image_vcpus)
+
+    parse_vcpu_option(guest, vcpus, image_vcpus)
+
+    conn = guest.conn
     if check_cpu:
         hostinfo = conn.getInfo()
         cpu_num = hostinfo[4] * hostinfo[5] * hostinfo[6] * hostinfo[7]
@@ -572,15 +736,9 @@ def get_vcpus(vcpus, check_cpu, guest, conn, image_vcpus=None):
             if not prompt_for_yes_or_no(msg, askmsg):
                 nice_exit()
 
-    if vcpus is None and image_vcpus is not None:
-        vcpus = int(image_vcpus)
-    if vcpus is not None:
-        try:
-            guest.vcpus = vcpus
-        except ValueError, e:
-            fail(e)
 
-def get_cpuset(cpuset, mem, guest, conn):
+def get_cpuset(cpuset, mem, guest):
+    conn = guest.conn
     if cpuset and cpuset != "auto":
         guest.cpuset = cpuset
 
@@ -597,6 +755,56 @@ def get_cpuset(cpuset, mem, guest, conn):
 
     return
 
+def parse_cpu(guest, optstring):
+    default_dict = {
+        "force": [],
+        "require": [],
+        "optional": [],
+        "disable": [],
+        "forbid": [],
+    }
+    model, opts = parse_optstr(optstring,
+                               basedict=default_dict,
+                               remove_first=True)
+
+    # Convert +feature, -feature into expected format
+    for key, value in opts.items():
+        policy = None
+        if value or len(key) == 1:
+            continue
+
+        if key.startswith("+"):
+            policy = "force"
+        elif key.startswith("-"):
+            policy = "disable"
+
+        if policy:
+            del(opts[key])
+            opts[policy].append(key[1:])
+
+    set_param = _build_set_param(guest.cpu, opts)
+    def set_features(policy):
+        for name in opts.get(policy):
+            guest.cpu.add_feature(name, policy)
+        del(opts[policy])
+
+    if model == "host":
+        guest.cpu.copy_host_cpu()
+        model = None
+
+    set_param("model", "model", model)
+    set_param("match", "match")
+    set_param("vendor", "vendor")
+
+    set_features("force")
+    set_features("require")
+    set_features("optional")
+    set_features("disable")
+    set_features("forbid")
+
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
 def get_network(net_kwargs, guest):
     n = VirtualNetworkInterface(**net_kwargs)
     guest.nics.append(n)
@@ -613,6 +821,9 @@ def set_os_variant(guest, distro_type, distro_variant):
             guest.set_os_variant(distro_variant)
 
 def parse_optstr_tuples(optstr):
+    """
+    Parse optstr into a list of ordered tuples
+    """
     optstr = str(optstr or "")
     optlist = []
 
@@ -625,28 +836,42 @@ def parse_optstr_tuples(optstr):
         opt_val = None
         if opt.count("="):
             opt_type, opt_val = opt.split("=", 1)
-            optlist.append((opt_type.lower(), opt_val.lower()))
+            optlist.append((opt_type.lower(), opt_val))
         else:
             optlist.append((opt.lower(), None))
 
     return optlist
 
-def parse_optstr(optstr, basedict=None):
+def parse_optstr(optstr, basedict=None, remove_first=False):
     """
     Helper function for parsing opt strings of the form
     opt1=val1,opt2=val2,...
-    'basedict' is a starting dictionary, so the caller can easily set
-    default values, etc.
+
+    @param basedict: starting dictionary, so the caller can easily set
+                     default values, etc.
+    @param remove_first: If true, remove the first options off the string
+                         and return it seperately. For example,
+                         --serial pty,foo=bar returns ("pty", {"foo" : "bar"})
 
     Returns a dictionary of {'opt1': 'val1', 'opt2': 'val2'}
     """
     optlist = parse_optstr_tuples(optstr)
     optdict = basedict or {}
+    first = None
+
+    if remove_first and optlist:
+        first_tuple = optlist[0]
+        if first_tuple[1] == None:
+            first = first_tuple[0]
+            optlist.remove(first_tuple)
 
     for opt, val in optlist:
-        optdict[opt] = val
+        if type(optdict.get(opt)) is list:
+            optdict[opt].append(val)
+        else:
+            optdict[opt] = val
 
-    return optdict
+    return (first, optdict)
 
 def parse_network_opts(conn, mac, network):
     net_type = None
@@ -675,7 +900,7 @@ def parse_network_opts(conn, mac, network):
         bridge_name = netdata
 
     # Pass the remaining arg=value pairs
-    opts = parse_optstr(",".join(args))
+    ignore, opts = parse_optstr(",".join(args))
     for opt_type, ignore_val in opts.items():
         if opt_type not in option_whitelist:
             fail(_("Unknown network option '%s'") % opt_type)
@@ -687,10 +912,10 @@ def parse_network_opts(conn, mac, network):
     # __init__
     if mac == "RANDOM":
         mac = None
-    return { "conn" : conn, "type" : net_type, "bridge": bridge_name,
-             "network" : network_name, "model" : model , "macaddr" : mac }
+    return {"conn" : conn, "type" : net_type, "bridge": bridge_name,
+            "network" : network_name, "model" : model , "macaddr" : mac}
 
-def digest_networks(conn, macs, bridges, networks, nics = 0):
+def digest_networks(conn, macs, bridges, networks, nics=0):
     macs     = listify(macs)
     bridges  = listify(bridges)
     networks = listify(networks)
@@ -713,12 +938,12 @@ def digest_networks(conn, macs, bridges, networks, nics = 0):
     if len(macs) > len(networks):
         fail(_("Cannot pass more mac addresses than networks."))
     else:
-        for dummy in range (len(macs),len(networks)):
+        for dummy in range(len(macs), len(networks)):
             macs.append(None)
 
     # Create extra networks up to the number of nics requested
     if len(macs) < nics:
-        for dummy in range(len(macs),nics):
+        for dummy in range(len(macs), nics):
             if User.current().has_priv(User.PRIV_CREATE_NETWORK, conn.getURI()):
                 net = _util.default_network(conn)
                 networks.append(net[0] + ":" + net[1])
@@ -734,22 +959,64 @@ def digest_networks(conn, macs, bridges, networks, nics = 0):
 
     return net_init_dicts
 
+def sanitize_keymap(keymap):
+    if not keymap:
+        return None
+
+    use_keymap = None
+
+    if keymap.lower() == "local":
+        use_keymap = virtinst.VirtualGraphics.KEYMAP_LOCAL
+
+    elif keymap.lower() != "none":
+        use_keymap = _util.check_keytable(keymap)
+        if not use_keymap:
+            raise ValueError(_("Didn't match keymap '%s' in keytable!") %
+                             keymap)
+
+    return use_keymap
+
+def parse_graphics(guest, optstring, basedict):
+    if optstring is None and not basedict:
+        return None
+
+    # Peel the model type off the front
+    gtype, opts = parse_optstr(optstring, basedict, remove_first=True)
+    if gtype == "none" or basedict.get("type") == "none":
+        return None
+    dev = VirtualGraphics(conn=guest.conn)
+
+    def set_param(paramname, dictname, val=None):
+        val = get_opt_param(opts, dictname, val)
+        if val == None:
+            return
+
+        if paramname == "keymap":
+            val = sanitize_keymap(val)
+        setattr(dev, paramname, val)
+
+    set_param("type", "type", gtype)
+    set_param("port", "port")
+    set_param("tlsPort", "tlsport")
+    set_param("listen", "listen")
+    set_param("keymap", "keymap")
+    set_param("passwd", "password")
+    set_param("passwdValidTo", "passwordvalidto")
+
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
+    return dev
+
 def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap,
-                 video_models, guest):
+                 video_models, graphics, guest):
     video_models = video_models or []
 
-    if ((vnc and nographics) or
-        (vnc and sdl) or
-        (sdl and nographics)):
-        raise ValueError, _("Can't specify more than one of VNC, SDL, "
-                            "or --nographics")
+    if graphics and (vnc or sdl or keymap or vncport or vnclisten):
+        fail(_("Cannot mix --graphics and old style graphical options"))
 
-    for model in video_models:
-        dev = virtinst.VirtualVideoDevice(guest.conn)
-        dev.model_type = model
-        guest.add_device(dev)
-
-    if not (vnc or nographics or sdl):
+    # If not graphics specified, choose a default
+    if not (vnc or nographics or sdl or graphics):
         if "DISPLAY" in os.environ.keys():
             logging.debug("DISPLAY is set: graphics defaulting to VNC.")
             vnc = True
@@ -757,39 +1024,38 @@ def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap,
             logging.debug("DISPLAY is not set: defaulting to nographics.")
             nographics = True
 
-    if nographics is not None:
-        guest.graphics_dev = None
-        return
+    if (sum(map(int, map(bool, [vnc, nographics, sdl, graphics])))) > 1:
+        raise ValueError(_("Can't specify more than one of VNC, SDL, "
+                           "--graphics or --nographics"))
 
-    # After this point, we are using graphics, so add a video device
-    # if one wasn't passed
+    # Build an initial graphics argument dict
+    basedict = {
+        "type"      : ((vnc and "vnc") or
+                       (sdl and "sdl") or
+                       (nographics and "none")),
+        "listen"    : vnclisten,
+        "port"      : vncport,
+        "keymap"    : keymap,
+    }
+
+    try:
+        dev = parse_graphics(guest, graphics, basedict)
+    except Exception, e:
+        fail(_("Error in graphics device parameters: %s") % str(e))
+
+    if not dev:
+        return
+    guest.graphics_dev = dev
+
+    # At this point we are definitely using graphics, so setup a default
+    # video card if necc.
     if not video_models:
-        guest.add_device(VirtualVideoDevice(conn=guest.conn))
-
-    if sdl is not None:
-        guest.graphics_dev = VirtualGraphics(type=VirtualGraphics.TYPE_SDL)
-        return
-
-    if vnc is not None:
-        guest.graphics_dev = VirtualGraphics(type=VirtualGraphics.TYPE_VNC)
-        if vncport:
-            guest.graphics_dev.port = vncport
-        if vnclisten:
-            guest.graphics_dev.listen = vnclisten
-
-    if keymap:
-        use_keymap = None
-
-        if keymap.lower() == "local":
-            use_keymap = virtinst.VirtualGraphics.KEYMAP_LOCAL
-
-        elif keymap.lower() != "none":
-            use_keymap = _util.check_keytable(keymap)
-            if not use_keymap:
-                raise ValueError(_("Didn't match keymap '%s' in keytable!") %
-                                 keymap)
-
-        guest.graphics_dev.keymap = use_keymap
+        video_models.append(None)
+    for model in video_models:
+        vdev = virtinst.VirtualVideoDevice(guest.conn)
+        if model:
+            vdev.model_type = model
+        guest.add_device(vdev)
 
 def get_sound(old_sound_bool, sound_opts, guest):
     if not sound_opts:
@@ -816,11 +1082,33 @@ def get_hostdevs(hostdevs, guest):
 ### Option parsing
 def check_before_store(option, opt_str, value, parser):
     if len(value) == 0:
-        raise OptionValueError, _("%s option requires an argument") %opt_str
+        raise OptionValueError(_("%s option requires an argument") % opt_str)
     setattr(parser.values, option.dest, value)
 
 def check_before_append(option, opt_str, value, parser):
     if len(value) == 0:
-        raise OptionValueError, _("%s option requires an argument") %opt_str
+        raise OptionValueError(_("%s option requires an argument") % opt_str)
     parser.values.ensure_value(option.dest, []).append(value)
 
+def get_opt_param(opts, dictnames, val=None):
+    if type(dictnames) is not list:
+        dictnames = [dictnames]
+
+    for key in dictnames:
+        if key in opts:
+            if val == None:
+                val = opts[key]
+            del(opts[key])
+
+    return val
+
+def partition(string, sep):
+    if not string:
+        return (None, None, None)
+
+    if string.count(sep):
+        splitres = string.split(sep, 1)
+        ret = (splitres[0], sep, splitres[1])
+    else:
+        ret = (string, None, None)
+    return ret
